import sys
import os

from . import Project
from . import User
from ..util.data import Auth
from ..util.data import BRSValues
from ..util.data import AlbCustomFields

try:

    import re
    import urllib
    import time
    import os.path
    import pprint
    import shutil

    try:
        import requests
    except:
        os.system('pip install requests')
        import requests

    try:
        from selenium import webdriver
    except:
        os.system('pip install selenium')
        from selenium import webdriver

    from selenium.webdriver.common.keys import Keys
    from selenium.webdriver.chrome.options import Options
    from selenium.webdriver.common.action_chains import ActionChains
    import json
    import datetime
    from inspect import getsourcefile
    from os.path import abspath
except ImportError as eIE:
    print(f'eIE occurred!')
    sys.exit(eIE)

def format_date_string_for_albatross(date_text,
                                     **kwargs):
    """
    Accepts a string of a date as grabbed from Loanpal, and formats it to be passed to Albatross API

    -Returns formatted date as a string if successful.
    -Returns -1 if there is an error

    :param test_mode_active:
    :param date_text:
    :return:
    """


    try:
        _date_input = date_text
        _date_with_dashes = str.replace(_date_input, '/', '-')
        _date_month = _date_with_dashes[0:2]
        _date_day = _date_with_dashes[3:5]
        _date_year = _date_with_dashes[6:8]

        _formatted_date = '20' + _date_year + '-' + _date_month + '-' + _date_day + 'T00:00:00Z'

        # print(_formatted_date)
        return _formatted_date

    except:
        return -1
def get_final_design_download_url(project_id=None,
                                  design_number=None,
                                  cust_info=None,
                                  **kwargs):
    """
    Retrieves the provided Final Design's download link from the given Project ID
    :param project_id:
    :param design_number:
    :param cust_info:
    :param test_mode_active:
    :param web_driver:
    :return:
    """

    # Open Project Details page for the customer
    driver.get(project_details_page)
    time.sleep(2.5)

    document_folder_icon_array = None
    try:
        document_folder_icon_array = driver.find_elements_by_xpath(
            "//*[@class='type text-center pb-0 col col-2']/div/div[1]")
    except Exception as e:
        print(f' The error {e} occurred when attempting to find hte final design folder icon in Albatross')
        return -5

    if document_folder_icon_array is not None:
        for folder_icon in document_folder_icon_array:
            print(folder_icon.text)

            if folder_icon.text == 'Final Design':
                final_design_folder_icon = folder_icon
                print('found the folder!')
                break
    else:
        time.sleep(0.5)
        driver.get(project_details_page)
        time.sleep(6)

        try:
            document_folder_icon_array = driver.find_elements_by_xpath(
                "//*[@class='type text-center pb-0 col col-2']/div/div[1]")

            for folder_icon in document_folder_icon_array:
                print(folder_icon.text)

                if folder_icon.text == 'Final Design':
                    final_design_folder_icon = folder_icon
                    print('found the folder!')
                    break

        except Exception as e:
            print(f' The error {e} occurred when attempting to find hte final design folder icon in Albatross')
            return -8

    parent_icon = final_design_folder_icon.find_element_by_xpath('..')
    parent_icon.click()
    time.sleep(1)

    # Create list of all files in this folder that have download links
    final_design_file_icons = driver.find_elements_by_css_selector("a[href*='7oak']")

    # Create regex expression to find correct file to download
    regex_str = r'(fd)?.*\S\s' \
                r'(final)?.*' \
                r'(design)?(desgin)?.*' \
                r'(' + design_number + r')?.*'

    # Create regex object for the Final Design string
    fd_regex = re.compile(regex_str, re.IGNORECASE)

    # Create list of words that will cause us to skip a file
    ignore_word_list = [
        r'shade',
        r'report',
        r'tool',
        r'dxf'
    ]

    # Create an array of regex objects for searching for these words in the button text
    word_check_array = []
    for word in ignore_word_list:
        word_check_array.append(re.compile(r"(?=" + word + r")", re.IGNORECASE))

    # Declare download_url variable
    download_url = None
    file_name = None
    file_path = None

    try:  # If the file name contains the the design number or the words design tool, then we will download the file.
        for button_found in final_design_file_icons:

            word_check_bool = False  # Start with false for each word, set to true if any of the ignore words are found
            for word_to_check in word_check_array:
                if word_to_check.search(button_found.text):
                    word_check_bool = True
                    break

            if word_check_bool is False and fd_regex.match(
                    button_found.text):  # If file has no ignored words and is FD:
                download_url = button_found.get_attribute("href")
                file_name = button_found.text + '.pdf'
                file_path = fdo.fd_download_location + '/' + file_name
                break

    except Exception as e:
        print(e)
        print("Failed to find target file among download icons")
        return -1

    if None in {download_url, file_name, file_path}:
        print('The Final Design was not found in the Final Design folder on the project. Cannot process this customer.')
        return -1

    else:
        time.sleep(1)

        return_list = [
            download_url,
            file_name,
            file_path,
            driver,
            _cust_info
        ]

        return return_list

def get_install_photos_download_url(project_id=None,
                                    cust_info=None,
                                    download_loc=None,
                                    **kwargs):
    """
    Retrieves the download link for the given Project ID's Install Photos
    :param cust_info:
    :param download_loc:
    :param test_mode_active:
    :param web_driver:
    :param project_id:
    :return:
    """

    # Open Project Details page for the customer
    driver.get(project_details_page)
    time.sleep(2.5)

    document_folder_icon_array = driver.find_elements_by_xpath(
        "//*[@class='type text-center pb-0 col col-2']/div/div[1]")

    install_photos_folder_icon = None
    for folder_icon in document_folder_icon_array:
        if folder_icon.text == 'Installation Photos':
            install_photos_folder_icon = folder_icon
            break

    if install_photos_folder_icon is None:
        return -1

    parent_icon = install_photos_folder_icon.find_element_by_xpath('..')
    parent_icon.click()
    time.sleep(1)

    # Create list of all files in this folder that have download links
    install_photos_file_icons = driver.find_elements_by_css_selector("a[href*='7oak']")
    download_url = None
    file_name = None
    file_path = None

    try:  # If the file name contains the proposal number, then we will download the file.

        for button_found in install_photos_file_icons:
            if ('photo' in button_found.text or 'Photo' in button_found.text):
                download_url = button_found.get_attribute("href")
                file_name = button_found.text + '.pdf'
                file_path = download_loc + '/' + file_name
                break

        if download_url is None or file_name is None or file_path is None:
            return -5
        else:
            _cust_info['install_photos_path'] = file_path

    except Exception as _e:
        print(_e)
        print(f'Failed to find target file among download icons - Error {_e}')
        return -1

    time.sleep(1)
    return_list = [download_url, file_name, file_path, driver, _cust_info]
    return return_list

def get_planset_download_url(project_id=None,
                             cust_info=None,
                             download_loc=None,
                             **kwargs):
    """
    Retrieves the provided Final Design's download link from the given Project ID
    :param cust_info:
    :param download_loc:
    :param test_mode_active:
    :param web_driver:
    :param project_id:
    :return:
    """

    # Open Project Details page for the customer
    driver.get(project_details_page)
    time.sleep(2.5)

    document_folder_icon_array = driver.find_elements_by_xpath(
        "//*[@class='type text-center pb-0 col col-2']/div/div[1]")

    for folder_icon in document_folder_icon_array:
        if folder_icon.text == 'Permit Pack':
            planset_folder_icon = folder_icon
            break

    parent_icon = planset_folder_icon.find_element_by_xpath('..')
    parent_icon.click()
    time.sleep(1)

    # Create list of all files in this folder that have download links
    planset_file_icons = driver.find_elements_by_css_selector("a[href*='7oak']")
    download_url = None

    try:  # If the file name contains the proposal number, then we will download the file.

        for button_found in planset_file_icons:
            if ('Pack' in button_found.text or 'PACK' in button_found.text) \
                    and 'Install' not in button_found.text \
                    and 'INSTALL' not in button_found.text \
                    and 'install' not in button_found.text \
                    and 'Tool' not in button_found.text \
                    and 'TOOL' not in button_found.text \
                    and 'tool' not in button_found.text \
                    and 'ENPHASE' not in button_found.text \
                    and 'enphase' not in button_found.text \
                    and 'Enphase' not in button_found.text \
                    and 'Brscad' not in button_found.text \
                    and 'brscad' not in button_found.text \
                    and 'BRSCAD' not in button_found.text:
                download_url = button_found.get_attribute("href")
                file_name = button_found.text + '.pdf'
                file_path = download_loc + '/' + file_name
                break
        if download_url is None or file_name is None or file_path is None:
            return -5
        else:
            _cust_info['planset_path'] = file_path


    except Exception as e:
        print(e)
        print("Failed to find target file among download icons")
        return -1

    time.sleep(1)
    return_list = [download_url, file_name, file_path, driver, _cust_info]
    return return_list

def update_final_design_sent_by_value(project_id=None,
                                      final_design_process_step=None,
                                      final_design_sent_by=29985,
                                      **kwargs):
    """
        Updates the "Final Design Sent By" custom field
        for the provided Project ID and Project Process Step ID,
        with the option to pass a user ID other than the default.

        29985 is the default user ID for who sent the Final Design,
        as it is the user ID for the Automation User.

        :param test_mode_active:
        :param project_id:
        :param final_design_process_step:
        :param final_design_sent_by:
        :return:
        """

    print(f'Updating "Final Design Sent By" custom field value . . .')

    #  Initialize variables
    try:
        # default_token_string = albatross_bearer_token
        bearer_token_string = Auth.albatross_non_expiring_token
        # print(bearer_token_string)
        _project_id = project_id
        _final_design_process_step = final_design_process_step
        _final_design_sent_by = final_design_sent_by
        session = requests.Session()
        session.headers.update({'Authorization': f'Bearer {bearer_token_string}'})

    except Exception as e:
        print('The following error occurred when attempting to initialize variables for updating the '
              'Final Design Sent By in Albatross: ' + e)

        return -1  # Error code for first try block

    try:
        params = [{
            'intValue': _final_design_sent_by,
            'dateValue': json.dumps(None),
            'timestampValue': json.dumps(None),
            'customFieldGroupAssignmentId': 20857,  # Final Design Sent By custom field group assignment ID
            'customFieldId': 10973  # Final Design Sent By custom field ID
        }]
    except Exception as e:
        print('While attempting to set the parameter dictionary for the POST to Albatross, the following error '
              'occurred: ' + e)

        return -2  # Error code for second try block

    try:
        session_response = session.post(
            f'https://albatross-api.myblueraven.com/api/v1/flow/customFieldValues/project/{str(_project_id)}'
            f'/processStep/{str(_final_design_process_step)}',
            headers={'Content-Type': 'application/json',
                     'Connection': 'close'
                     },
            json=params
        )
        print(f'Final Design Sent By updated: {session_response}')
    except Exception as e:
        print('While attempting to update the Final Design Sent By custom field, the following error occurred: ' + e)

        return -3  # Error code for third try block

    # return _financial_agreement_signed_date    # Not sure which one I like more
    return 1  # Returning 1 indicates the function ran successfully


def update_financial_agreement_signed_value(project_id,
                                            final_design_process_step,
                                            financial_agreement_signed_date,
                                            test_mode_active=False):
    """
    Updates the "Financial Agreement Signed" custom field
    for the provided Project ID and Project Process Step ID.

    :param test_mode_active:
    :param project_id:
    :param final_design_process_step:
    :param financial_agreement_signed_date:
    :return:
    """

    print(f'Updating "Financial Agreement Signed Date" custom field value . . .')

    #  Initialize variables
    try:
        bearer_token_string = non_expiring_token
        _project_id = project_id
        _final_design_process_step = final_design_process_step
        _financial_agreement_signed_date = financial_agreement_signed_date  # TODO - add a way of making sure its the right formatting
        session = requests.Session()
        session.headers.update({'Authorization': f'Bearer {bearer_token_string}'})

    except Exception as e:
        print('The following error occurred when attempting to initialize variables for updating the '
              'Financial Agreement Signed Date in Albatross: ' + e)

        return -1  # Error code for first try block

    try:
        _formatted_date = format_date_string_for_albatross(_financial_agreement_signed_date)
        if _formatted_date == -1:
            return -1.5
    except Exception as e:
        print(f'While attempting to format the date, the following error occurred: {e}')
        return -1.75

    try:
        params = [{
            'intValue': json.dumps(None),
            'dateValue': _formatted_date,
            'timestampValue': json.dumps(None),
            'customFieldGroupAssignmentId': 19505,  # Financial Agreement Send date custom field group assignment ID
            'customFieldId': 108  # Financial Agreement Send date custom field ID
        }]
    except Exception as e:
        print('While attempting to set the parameter dictionary for the POST to Albatross, the following error '
              'occurred: ' + e)

        return -2  # Error code for second try block

    try:
        session_response = session.post(
            f'https://albatross-api.myblueraven.com/api/v1/flow/customFieldValues/project/{_project_id}'
            f'/processStep/{_final_design_process_step}',
            headers={'Content-Type': 'application/json'},
            json=params
        )
        print(f'Financial Agreement Signed by Homeowner Date updated: {session_response}')
    except Exception as e:
        print(
            'While attempting to update the Financial Agreement Signed Date custom field, the following error occurred: ' + e)

        return -3  # Error code for third try block

    # return _financial_agreement_signed_date    # Not sure which one I like more
    return 1  # Returning 1 indicates the function ran successfully


def update_financial_agreement_sent_value(project_id,
                                          final_design_process_step,
                                          financial_agreement_sent_date,
                                          test_mode_active=False):
    """
    Updates the "Financial Agreement Sent to Homeowner" custom field
    for the provided Project ID and Project Process Step ID.

    :param test_mode_active:
    :param project_id:
    :param final_design_process_step:
    :param financial_agreement_sent_date:
    :return:
    """

    print(f'Updating "Financial Agreement Sent" custom field value . . .')

    #  Initialize variables
    try:
        bearer_token_string = non_expiring_token
        _project_id = project_id
        _final_design_process_step = final_design_process_step
        _financial_agreement_sent_date = financial_agreement_sent_date  # TODO - add a way of making sure its the right formatting
        session = requests.Session()
        session.headers.update({'Authorization': f'Bearer {bearer_token_string}'})

    except Exception as e:
        print('The following error occurred when attempting to initialize variables for updating the '
              'Financial Agreement Sent Date in Albatross: ' + e)

        return -1  # Error code for first try block

    try:
        _formatted_date = format_date_string_for_albatross(_financial_agreement_sent_date)
        if _formatted_date == -1:
            return -1.5
    except Exception as e:
        print(f'While attempting to format the date, the following error occurred: {e}')
        return -1.75

    try:
        params = [{
            'intValue': json.dumps(None),
            'dateValue': _formatted_date,
            'timestampValue': json.dumps(None),
            'customFieldGroupAssignmentId': 19503,  # Financial Agreement Send date custom field group assignment ID
            'customFieldId': 107  # Financial Agreement Send date custom field ID
        }]
    except Exception as e:
        print('While attempting to set the parameter dictionary for the POST to Albatross, the following error '
              'occurred: ' + e)

        return -2  # Error code for second try block

    try:
        session_response = session.post(
            f'https://albatross-api.myblueraven.com/api/v1/flow/customFieldValues/project/{_project_id}'
            f'/processStep/{_final_design_process_step}',
            headers={'Content-Type': 'application/json'},
            json=params
        )
        print(f'Financial Agreement Sent to Homeowner Date updated: {session_response}')
    except Exception as e:
        print(
            'While attempting to update the Financial Agreement Sent Date custom field, the following error occurred: ' + e)

        return -3  # Error code for third try block

    # return _fd_sent_date     # Not sure which one I like more
    return 1  # Returning 1 indicates the function ran successfully


def update_final_design_sent_to_homeowner_value(project_id,
                                                final_design_process_step,
                                                fd_sent_timestamp='Not provided',
                                                test_mode_active=False):
    """
    Updates the "Final Design Sent to Homeowner" custom field for the provided Project ID and Project Process Step ID.
    You can pass a date as a third argument, or let it default to using the current time.
    :param test_mode_active:
    :param project_id:
    :param final_design_process_step:
    :param fd_sent_timestamp:
    :return:
    """

    print(f'Updating "Final Design Sent to Homeowner" custom field value . . .')

    print(f'the kwargs for the Albatross.update_final_design_sent_by is {str(project_id)} for project id and {str(final_design_process_step)} for FD process step')
    #  Initialize variables
    try:
        bearer_token_string = Auth.albatross_non_expiring_token
        _project_id = project_id
        _final_design_process_step = final_design_process_step
        print(bearer_token_string + '\n' + str(_project_id) + '\n' + str(_final_design_process_step))
        if fd_sent_timestamp == 'Not provided':
            # print('Final Design timestamp not provided to function')
            _current_time = datetime.datetime.utcnow()
            _now_timestamp = datetime.datetime.isoformat(_current_time, sep='T', timespec='auto', )
            _formatted_timestamp = _now_timestamp
        else:
            # print('Final Design timestamp provided to function')
            _fd_sent_timestamp = fd_sent_timestamp
            _formatted_timestamp = _fd_sent_timestamp  # TODO - add a way of making sure its the right formatting

        session = requests.Session()
        session.headers.update({'Authorization': f'Bearer {bearer_token_string}'})

    except Exception as e:
        print('The following error occurred when attempting to initialize variables for updating the Final Design'
              ' Sent Date in Albatross: ' + e)
        return -1  # Error code for first try block

    try:
        params = [{
            'intValue': json.dumps(None),
            'dateValue': json.dumps(None),
            'timestampValue': _formatted_timestamp,
            'customFieldGroupAssignmentId': 19502,  # FD Send date custom field group assignment ID
            'customFieldId': 103  # FD Send date custom field ID
        }]
    except Exception as e:
        print('While attempting to set the parameter dictionary for the POST to Albatross, the following error '
              'occurred: ' + e)
        return -2  # Error code for second try block

    try:
        session_response = session.post(
            f'https://albatross-api.myblueraven.com/api/v1/flow/customFieldValues/project/{str(_project_id)}'
            f'/processStep/{str(_final_design_process_step)}',
            headers={'Content-Type': 'application/json',
                     'Connection': 'close'
                     },
            json=params
        )
        print(f'Final Design Sent to Homeowner Date updated: {session_response}')
    except Exception as e:
        print('While attempting to update the Final Design Sent Date custom field, the following error occurred: ' + e)
        return -3  # Error code for third try block

    # return _fd_sent_date     # Not sure which one I like more
    return 1  # Returning 1 indicates the function ran successfully


# TODO - TO TEST-Test the click the action button function
def click_pending_approval_action_button(project_id,
                                         contact_id,
                                         final_design_process_step,
                                         web_driver=None,
                                         test_mode_active=False):
    """
    Attempts to press the "Design Sent, Pending Approval" action button on the Design
    and Financing Process Step that corresponds to the passed project_id and process_step_id.

    :param contact_id:
    :param web_driver:
    :param test_mode_active:
    :param project_id:
    :param final_design_process_step:
    :return:
    """

    print(
        f'Now attempting to click the "Design Sent, Pending Approval" action button on the project process step . . .')

    try:
        # Set Chrome Driver Options
        chromeOptions = Options()
        if windowless_chrome_mode is True:
            chromeOptions.add_argument("--headless")
            chromeOptions.add_argument("--disable-gpu")
        chromeOptions.add_argument('no-sandbox')
        chromeOptions.add_argument("--start-maximized")
        chromeOptions.add_argument("--window-size=1920x1080")
        chromeOptions.add_experimental_option("prefs", {
            "download.default_directory": fdo.fd_download_location,
            "download.prompt_for_download": False,
            "download.directory_upgrade": True,
            "safebrowsing_for_trusted_sources_enabled": False,
            "safebrowsing.enabled": False
        })

        # Set necessary variables
        login_url = br_options.albatross_login_url
        user_name = br_options.albatross_user_name
        password = br_options.albatross_password
        #  project_details_page = "https://albatross.myblueraven.com/project/{}/details".format(str(project_id))
        design_and_financing_process_step_page = f'https://albatross.myblueraven.com/project/{project_id}/processStep' \
                                                 f'/{final_design_process_step}?processStepId=3355&contactId={contact_id}'
    except Exception as e:
        print(f'While initializing variables before attempting to click the Design Sent, Pending '
              f'Approval action button, the following error occurred: {e}')
        return -1

    try:
        # Initialize headless Chrome driver
        driver = webdriver.Chrome(executable_path=fdo.chrome_driver_path,
                                  options=chromeOptions,
                                  keep_alive=False)

        # Login to Albatross
        driver.get(login_url)
        loginfield = driver.find_element_by_name('login')
        loginfield.send_keys(user_name)
        passwordfield = driver.find_element_by_id('password')
        passwordfield.send_keys(password)
        passwordfield.send_keys(Keys.RETURN)
        time.sleep(1)
    except Exception as e:
        print(f'While loading Albatross and logging in, the following error occurred: {e}')
        return -2

    try:
        # Open Project Details page for the customer
        driver.get(design_and_financing_process_step_page)
        time.sleep(2)
    except Exception as e:
        print(f'While attempting to load the Design and Financing process step web page, the following error '
              f'occurred: {e}')
        return -3

    try:
        load_time = 0
        load_time_limit = 6
        while load_time < load_time_limit:
            try:
                pending_approval_action_button = driver.find_element_by_xpath(
                    "//*[contains(text(), 'Design Sent, Pending Approval')]"
                )
                pending_approval_action_button.click()
                time.sleep(1)
                load_time += 100
            except:
                print('Waiting for action button to load . . .')
                load_time += 1
                time.sleep(1)
        if load_time == load_time_limit:
            print('Timed out while waiting for action button to appear!')
            return -3.75
    except Exception as e:
        print(f'While attempting to find and click the Design Sent, Pending Approval action button, '
              f'the following error occurred: {e}')
        return -4

    _return_dictionary = {
        'outcome_code': 1,
        'web_driver_object': driver
    }

    return _return_dictionary


def download_file_from_link(file_url,
                            file_name,
                            testing_mode_active=False,
                            web_driver=None,
                            download_location=os.path.abspath(os.path.dirname(sys.argv[0]))):
    """Downloads the file from a provided link to the specified file path location, with the specified file name

    :param testing_mode_active:
    :param web_driver:
    :param file_url:
    :param file_name:
    :param download_location:
    :return:
    """
    print('Downloading ' + file_name + ' from fetched URL . . .')

    # Set necessary variables
    try:
        _download_location = download_location

        if _download_location[-1:] != '\\':
            _download_location = _download_location + '\\'

        _file_url = file_url
        _file_name = file_name
        _login_url = br_options.albatross_login_url
        _user_name = br_options.albatross_user_name
        _password = br_options.albatross_password
        _payload = {
            'login': _user_name,
            'password': _password
        }

        downloaded_file_path = _download_location + _file_name

    except Exception as e:
        print(
            f'While initializing variables to download the file from the provided URL, the following error occurred: {e}')
        return -1

    # Download the final design
    try:
        with requests.Session() as _session:
            _post = _session.post(_login_url, data=_payload)
            _r = _session.get(_file_url, allow_redirects=True)
            open(_download_location + _file_name, 'wb').write(_r.content)
        time.sleep(1)
    except Exception as e:
        print(f'While attempting to download the file from the provided URL, the following error occurred: {e}')
        return -2

    try:
        # Wait for file to download
        download_wait_time = 0
        while download_wait_time < 15 and os.path.isfile(downloaded_file_path) is False:
            download_wait_time += 1
            print('Waiting for download - time elapsed = ' + str(download_wait_time))
            time.sleep(1)

        # Return error if file wasn't downloaded
        if os.path.isfile(downloaded_file_path) is True:

            return downloaded_file_path
        else:
            return -3
    except Exception as e:
        print(f'While waiting to download the file from the provided URL, the following error occurred: {e}')
        return -4


def leave_account_note(project_id,
                       contact_id,
                       note_title,
                       note_body,
                       web_driver=None,
                       test_mode_active=False):
    """
    Leaves a note on the provided account with the provided note title and note body.

    :param web_driver:
    :param test_mode_active:
    :param project_id:
    :param contact_id:
    :param note_title:
    :param note_body:
    :return:
    """

    print('Now leaving account note . . .')

    # Set necessary variables
    _login_url = br_options.albatross_password
    _user_name = br_options.albatross_user_name
    _password = br_options.albatross_password
    _project_details_page = "https://albatross.myblueraven.com/project/{}/details".format(str(project_id))
    _albatross_save_note_button_xpath = '#project-notes-container > div > div > div > div.px-3.elevation-0.square-' \
                                        'card.v-card.v-sheet.theme--light > div.text-left.mb-2 > button.white--text.' \
                                        'v-btn.v-btn--contained.theme--light.v-size--default.primaryCustom > span'
    _albatross_newest_note_xpath = '#project-notes-container > div > div > div > div.v-data-table.elevation-0' \
                                   '.mt-1.theme--light > div > table > tbody > tr:nth-child(1) > td.text-right' \
                                   ' > button > span > i'
    _albatross_add_comment_to_note_xpath = '//*[contains(text(), "Add Comment")]'

    if web_driver is None:
        # If no web driver object passed, initialize and log in
        try:
            _contact_id_UNUSED = contact_id
            # Set Chrome Driver Options
            chromeOptions = Options()
            if windowless_chrome_mode is True:
                chromeOptions.add_argument("--headless")
                chromeOptions.add_argument("--disable-gpu")
            chromeOptions.add_argument("--start-maximized")
            chromeOptions.add_argument('no-sandbox')
            chromeOptions.add_argument("--window-size=1920x1080")
            chromeOptions.add_experimental_option("prefs", {
                "download.default_directory": fdo.fd_download_location,
                "download.prompt_for_download": False,
                "download.directory_upgrade": True,
                "safebrowsing_for_trusted_sources_enabled": False,
                "safebrowsing.enabled": False
            })

        except Exception as e:
            print(f'While attempting to declare variables for leaving a note, the following error occurred: {e}')
            # TODO - LOG ERROR
            return -1

        try:
            ### Initialize headless Chrome driver
            _driver = webdriver.Chrome(
                executable_path=fdo.chrome_driver_path,
                options=chromeOptions,
                keep_alive=False)

            # Login to Albatross
            _driver.get(_login_url)
            time.sleep(3)
            loginfield = _driver.find_element_by_name('login')
            loginfield.send_keys(_user_name)
            passwordfield = _driver.find_element_by_id('password')
            passwordfield.send_keys(_password)
            passwordfield.send_keys(Keys.RETURN)
            time.sleep(2.5)
        except Exception as e:
            print(f'While attempting to start a headless chrome driver and login to Albatross, the following error '
                  f'occurred: {e}')
            # TODO - LOG ERROR
            return -2
    else:
        # If web driver object provided, try using that
        try:
            _driver = web_driver
            print('Web driver passed to function has been set as the driver')
        except Exception as e:
            print(f'While attempting to set the passed web driver as the web driver for leaving a note, the '
                  f'following error occurred: {e}')
            # TODO - LOG ERROR
            return -2.5

    try:
        # Open Project Details page for the customer
        _driver.get(_project_details_page)
        time.sleep(2)
    except Exception as e:
        print(f'While attempting to ACTION the following error occurred: {e}')
        # TODO - LOG ERROR
        return -2.75

    try:
        # Select the notes entry box
        _driver.find_element_by_xpath('//*[@id="project-container"]/div/div[2]/div[2]/'
                                      'header/div/div[2]/div/div/div[2]/div/div[3]').click()
    except Exception as e:
        print(f'While attempting to click the note entry box, the following error occurred: {e}')
        # TODO - LOG ERROR
        return -3
    try:
        # Enter note title
        note_title_field = _driver.find_elements_by_xpath('//textarea[contains(@id, "input-")]')
        note_title_field[-1].send_keys(note_title)
    except Exception as e:
        print(f'While attempting to enter the note title into the note input box, the following error occurred: {e}')
        # TODO - LOG ERROR
        return -4

    try:
        # Click save note (title) button
        save_note_title_button = _driver.find_element_by_css_selector(_albatross_save_note_button_xpath)
        save_note_title_button.click()
        time.sleep(2)
    except Exception as e:
        print(f'While attempting to click the Save Note button, the following error occurred: {e}')
        # TODO - LOG ERROR
        return -5

    try:
        # Click note
        three_buttons_option = _driver.find_element_by_css_selector(_albatross_newest_note_xpath)
        three_buttons_option.click()
        time.sleep(1.5)
    except Exception as e:
        print(f'While attempting to click the button to show the add note option for a comment,'
              f' the following error occurred: {e}')
        # TODO - LOG ERROR
        return -6

    try:
        # Click button to add comments to note
        add_note_to_comment_button = _driver.find_element_by_xpath(_albatross_add_comment_to_note_xpath)
        add_note_to_comment_button.click()
        time.sleep(1.5)
    except Exception as e:
        print(f'While attempting to click the Add Comment button, the following error occurred: {e}')
        # TODO - LOG ERROR
        return -7

    try:
        # Enter note text and save
        comment_array = _driver.find_elements_by_xpath('//*[contains(@id, "input-")]')
        comment_array[-1].send_keys(note_body)
        save_comment_button = _driver.find_elements_by_xpath('//*[contains(text(), "Save")]')[-1]
        save_comment_button.click()
        time.sleep(0.5)
    except Exception as e:
        print(f'While attempting to enter the comment note text and save it, the following error occurred: {e}')
        # TODO - LOG ERROR
        return -8

    # Returns 1 if everything runs successfully
    return_dictionary = {
        'outcome_code': 1,
        'web_driver_object': _driver
    }

    return return_dictionary


if __name__ == '__main':
    print('Albatross main module initialized!')