import sys
import json
import datetime
import requests

try:
    from .. import db_conn
    from ...tools.find_best_match import find_best_match
except ImportError as eI:
    from brs_tk.Albatross import db_conn
    from brs_tk.tools.find_best_match import find_best_match
except Exception as e:
    print(f'Unexpected exception {e} while attempting to import the db_conn module for the Albatross.Project module')

class Project:

    def __init__(self, **kwargs):
        self.project_attachments = []
        self.project_process_steps = []
        self.custom_field_values = []

        if kwargs.get('project_id', False) is False:
            raise TypeError('No project ID was provided when declaring the project object')
        else:
            self.project_id = kwargs.get('project_id', False)
        
        for (key, pair) in kwargs.items():
            self.__setattr__(key, pair)

        if kwargs.get('project_name', False) is False:
            self.load_basic_project_info(**kwargs)

    def load_basic_project_info(self, **kwargs):
        _basic_project_info_query_str = """
        WITH
            d_and_f_proposal_number as (
                SELECT 
                    pps.project_id "project_id",	
                    pps.id "project_process_step_id",		
                    plh.proposal_nbr as "proposal_number",
                    cf.field_name "cf_field_name",
                    cf.id "cf_id",
                    cfga.id "cfga_id"
                                   
                FROM 
                    flow.project_process_step_custom_field_value ppscfv
                
                inner join flow.project_process_step pps on pps.id = ppscfv.project_process_step_id
                inner join flow.process_step ps on ps.id = pps.process_step_id
                inner join flow.custom_field_group_assignment cfga on cfga.id = ppscfv.custom_field_group_assignment_id
                inner join flow.custom_field cf on cf.id = cfga.custom_field_id
                inner join brs.proposal_log_history plh on plh.project_id = pps.project_id and plh.id = ppscfv.int_value
                
                
                WHERE 
                    ps.id = 3355
                    and 
                    cf.id = 622
            )
            ,
            
            d_and_f_design_number as (
                SELECT 
                    pps.project_id "project_id",	
                    pps.id "project_process_step_id",		
                    dlh.reference_nbr as "design_number",
                    cf.field_name "cf_field_name",
                    cf.id "cf_id",
                    cfga.id "cfga_id"                   
                
                FROM 
                    flow.project_process_step_custom_field_value ppscfv
                
                inner join flow.project_process_step pps on pps.id = ppscfv.project_process_step_id
                inner join flow.process_step ps on ps.id = pps.process_step_id
                inner join flow.custom_field_group_assignment cfga on cfga.id = ppscfv.custom_field_group_assignment_id
                inner join flow.custom_field cf on cf.id = cfga.custom_field_id
                inner join brs.design_log_history dlh on dlh.project_id = pps.project_id and dlh.id = ppscfv.int_value
                
                
                where 
                    ps.id = 3355
                    and 
                    cf.id = 9109
            )
                    
        SELECT
            pj.project_name "project_name",
            pj.street1 "street_address1",
            pj.street2 "street_address2",
            pj.city "city",
            pj.postal_code "zip_code",
            pj.contact_id "contact_id",
            dafdn.design_number "design_number",
            dafpn.proposal_number "proposal_number"
        
        FROM
            flow.project pj
        
        inner join d_and_f_proposal_number dafpn on 
            dafpn.project_id = pj.id
        
        inner join d_and_f_design_number dafdn on
            dafdn.project_id = pj.id
        
        WHERE
            pj.id = """ + str(self.project_id)

        _basic_info_response = db_conn.post_query(query_string=_basic_project_info_query_str)[0]
        _basic_info_columns = [
            'project_name', 'street_address1', 'street_address2',
            'city', 'zip_code', 'contact_id', 'design_number',
            'proposal_number'
        ]
        for x in range(0,len(_basic_info_columns)):
            self.__setattr__(_basic_info_columns[x], _basic_info_response[x])




    def find_final_design(self, return_type='string'):



        print('trying to find the final design')
        print(self.project_attachments)

        # Words to exclude a file name from the match
        words_to_ignore = [
            ('shade', False),
            ('tool', False),
            ('dxf', False),
            ('SS', True),
            ('pvwatts', False)
        ]

        _attachment_file_names = []
        for _attachment in self.project_attachments:
             if _attachment.file_folder == 'Final Design':
                _attachment_file_names.append(_attachment.file_name)
                print(_attachment)

        # For no matches, normally migrated documents fodler from older projects migrated from Zendesk
        if _attachment_file_names == []:
            _attachment_file_names = [att.file_name for att in self.project_attachments]

        print(f'list of attachment files names {_attachment_file_names}')

        if hasattr(self, 'project_name'):
            sample_string_to_match1 = self.project_name
            sample_string_to_match2 = self.project_name
        else:
            self.load_basic_project_info()
            sample_string_to_match1 = self.project_name
            sample_string_to_match2 = self.project_name

        if not (hasattr(self, 'design_number') or hasattr(self, 'proposal_number')):
            raise TypeError('NoneType cannot be both design and proposal number. Required values!')

        if hasattr(self, 'design_number'):
            sample_string_to_match1 = 'FD Final Design ' + str(self.design_number) + ' ' + self.project_name + '.pdf'

        if hasattr(self, 'proposal_number'):
            sample_string_to_match2 = 'FD Final Design ' + str(self.proposal_number) + ' ' + self.project_name + '.pdf'

        print(f'Sample string 1 to match: {sample_string_to_match1}')
        print(f'Sample string 2 to match: {sample_string_to_match2}')

        match_result1 = find_best_match(stringToMatch=sample_string_to_match1,
                                        listToCheck=_attachment_file_names,
                                        string_ignore=words_to_ignore)

        match_result2 = find_best_match(stringToMatch=sample_string_to_match2,
                                        listToCheck=_attachment_file_names,
                                        string_ignore=words_to_ignore)

        print(f' the match result1 is {match_result1}')
        print(f' the match result2 is {match_result2}')

        # This goes with the one matching the match_result_1 unless it is way less of a match
        print(f'The proposal number file matched {str(match_result2[1])}, '
              f'and the design number file matched {str(match_result1[1])} ')
        print(f'The score difference between design number and proposal number match is {(match_result1[1] - match_result2[1])}')

        if match_result1[1] - match_result2[1] < -33:
            if return_type=='string':
                return match_result2[0]
            elif return_type=='match_object':
                return match_result2
            else:
                for pj_att in self.project_attachments:
                    if pj_att.file_name == match_result2[0]:
                        return pj__att
                return False

        else:
            if return_type=='string':
                return match_result1[0]
            elif return_type=='match_object':
                return match_result1
            else:
                for pj_att in self.project_attachments:
                    if pj_att.file_name == match_result1[0]:
                        return pj_att
                return False




    def list_all_attachments(self):

        project_all_attachment_info_query_string = """
                select 
                    att.filename "file_name",
                    attTyp.attachment_type "file_folder",
                    att.s3_key "s3_key",	
                    att.content_type "file_type",
                    pa.project_id "project_id",
                    pa.date_created "date_created"            

                from 
                    flow.project_attachment pa 

                inner join flow.attachment att on
                    att.id = pa.attachment_id

                inner join flow.attachment_type attTyp on
                    attTyp.id = att.attachment_type_id        

                where 
                    pa.project_id = """ + str(self.project_id) + """
                UNION
                
                select 
                    att.filename "file_name",
                    attTy.attachment_type "file_folder",
                    att.s3_key "s3_key",	
                    att.content_type "file_type",
                    pps.project_id "project_id",
                    ppsa.date_created "date_created"            
                
                from 
                    flow.project_process_step_attachment ppsa
                
                inner join flow.attachment att on
                    att.id = ppsa.attachment_id
                
                inner join flow.attachment_type attTy on
                    attTy.id = att.attachment_type_id        
                
                inner join flow.project_process_step pps on pps.id = ppsa.project_process_step_id
                
                where 
                    pps.project_id = """ + str(self.project_id)


        all_attachments_response = db_conn.post_query(query_string=project_all_attachment_info_query_string)
        print(all_attachments_response)

        download_url_prefix = r"https://7oaks-albatross.s3.amazonaws.com/"

        for attachment_info_entry in all_attachments_response:
            self.project_attachments.append(
                ProjectAttachment(
                    file_name= attachment_info_entry[0],
                    file_folder= attachment_info_entry[1],
                    s3_key= attachment_info_entry[2],
                    file_type= attachment_info_entry[3],
                    project_id= attachment_info_entry[4],
                    date_created= attachment_info_entry[5],
                    attachment_download_link= download_url_prefix + attachment_info_entry[2]
                )
            )
            print('Attached File - ' + attachment_info_entry[0] + '\nFolder Location - ' + attachment_info_entry[1] + '\n')

        return self.project_attachments

class ProjectAttachment:

    def __init__(self, **kwargs):

        if kwargs.get('file_name', None) is None and kwargs.get('file_type', None) is None:
            raise TypeError('No file name provided for the attachment, nor a file type.')

        elif not kwargs.get('file_name', None):
            self.file_name = kwargs.get('file_name')

        else:
            self.file_type = kwargs.get('file_type')

        self.s3_key = kwargs.get('s3_key', None)
        self.download_location = kwargs.get('download_location', None)
        self.attachment_download_link = kwargs.get('attachment_download_link', None)

        for (key, pair) in kwargs.items():
            self.__setattr__(key, pair)

    def __str__(self):
        return_str = ''
        return_str += 'Attached File - ' + self.file_name + '\n'
        return_str += 'Folder Location - ' + self.file_folder + '\n'
        return_str += 'Download Link - ' + self.get_download_link() + '\n'
        return_str += 'File Type - ' + self.file_type + '\n'

        return return_str



    def fetch_attachment_info(self, **kwargs):

        attachment_info_query_string = """
        select 
            att.filename "file_name",
            attTyp.attachment_type "file_folder",
            att.s3_key "s3_key",	
            att.content_type "file_type",
            pa.project_id "project_id",
            pa.date_created "date_created"            
        
        from 
            flow.project_attachment pa 
            
        inner join flow.attachment att on
            att.id = pa.attachment_id
        
        inner join flow.attachment_type attTyp on
            attTyp.id = att.attachment_type_id        
            
        where 
            pa.project_id = """ + str(self.project_id) + """
            and
            att.filename = '""" + self.file_name + """'"""

        attachment_info_response = db_conn.post_query(query_string=attachment_info_query_string)[0]
        self.s3_key = attachment_info_response[2]
        self.file_folder = attachment_info_response[1]
        self.file_type = attachment_info_response[3]
        self.date_created = attachment_info_response[4]

        # print(attachment_info_response)

    def get_download_link(self):

        try:
            if self.attachment_download_link != None:
                return self.attachment_download_link
        except ImportError as eI:
            pass
        except Exception as e:
            print(f'Unexpected exception {e} occurred')

        if None in (self.file_name, self.project_id):
            raise TypeError('Must have a value for the file name and project ID to be able to download it')

        download_url_prefix = r"https://7oaks-albatross.s3.amazonaws.com/"

        if None in (self.s3_key, self.download_location):
            self.fetch_attachment_info()

        # Append the S3 key to the albatross website handle
        if self.s3_key == None:
            self.attachment_download_link = None
            raise TypeError('We need a value for the s3 key to download it!')
        else:
            self.attachment_download_link = download_url_prefix + self.s3_key

        return self.attachment_download_link

    # def find_final_design(self, **kwargs):
    #

    def download(self, **kwargs):
        # self.download_location = kwargs.get('download_location', None)
        #
        #
        #
        # if None in (self.file_name, self.project_id):
        #     raise TypeError('Must have a value for the file name and project ID to be able to download it')
        #
        # download_url_prefix = r"https://7oaks-albatross.s3.amazonaws.com/"
        #
        # if None in (self.s3_key, self.download_location):
        #     self.fetch_attachment_info()
        #
        # # Append the S3 key to the albatross website handle
        # if self.s3_key == None:
        #     self.attachment_download_link = None
        #     raise TypeError('We need a value for the s3 key to download it!')
        # else:
        #     self.attachment_download_link = download_url_prefix + self.s3_key
        #
        # print(f'Attachment download link: {self.attachment_download_link}')
        if self.download_location == None:

            from brs_tk.util.data.BRSValues import DriverConfigs
            self.download_location = DriverConfigs.default_download_location

            from pathlib import Path
            Path(self.download_location).mkdir(parents=True, exist_ok=True)

        if self.attachment_download_link == None:
            self.attachment_download_link = self.get_download_link()

        # Download file and write to download location
        with requests.Session() as _session:
            _get = _session.get(self.attachment_download_link, allow_redirects=True)
            open(str.replace(self.download_location,'/','\\') + '\\' + self.file_name, 'wb').write(_get.content)

        self.update_property('final_design_file_path', str.replace(self.download_location,'/','\\') + '\\' + self.file_name)

        return str.replace(self.download_location,'/','\\') + '\\' + self.file_name

    def update_property(self, key, pair):
        """
        Updates the value for the specified property key / property name
        """

        if key is None:
            raise TypeError('A none-type value was provided for the property name.')

        else:
            self.__setattr__(key, pair)
    
class ProjectProcessStep:

    def __init__(**kwargs):

        if kwargs.get('project_process_step_id', None) is None:
            raise TypeError('No project process step ID provided')
        else:
            self.project_process_step_id = kwargs.get('project_process_step_id', None)

        if kwargs.get('project_id', None) is None:
            self.project_id = 0
        else:
            self.project_id = kwargs.get('project_id', None)

        for (key, pair) in kwargs.items():
            self.__setattr__(key, pair)

    def get_project_id(self):

        get_project_id_from_project_process_step_id_query_string = """
        select 
            pps.project_id
        from 
            flow.project_process_step pps
        where 
            pps.id = """ + str(self.project_process_step_id)


        _project_id = db_conn.post_query(query_string=get_project_id_from_project_process_step_id_query_string)[0]
        self.project_id = _project_id


    # def get_status(self):

class ProjectProcessStepCustomFieldValue:

    def __init__(self, **kwargs):

        if kwargs.get('project_process_step_custom_field_value_id', None) is None:
            raise TypeError('Must provide a project process step custom field value id')
        else:
            self.project_process_step_custom_field_value_id = kwargs.get('project_process_step_custom_field_value_id', None)

        for (key, pair) in kwargs.items():
            self.__setattr__(key, pair)

if __name__ == '__main__':
    #sys.exit(0)
    # args_dict = {}
    # args_dict['file_name'] = 'huston shade report.pdf'
    # args_dict['project_id'] = 423290
    # huston_shade_report = ProjectAttachment(file_name="huston shade report.pdf", project_id=423290)
    #
    # huston_shade_report.fetch_attachment_info()
    #
    # huston_shade_report.download(download_location=r"C:/Users/Christopher Bahr/Desktop/")
    #
    test_project = Project(project_id=349226)
    #
    all_atts = test_project.list_all_attachments()
    print(all_atts)
    # for entry in all_atts:
    fd_attachment_obj = test_project.find_final_design(return_type='attachment object')
    print(fd_attachment_obj)
    print(fd_attachment_obj.download())
    print(fd_attachment_obj.final_design_file_path)
    # # tamara_washington = Project({'project_id': 387372})
    # test_process_step = ProjectProcessStep({'project_process_step_id':3499819})
    # print(test_process_step.project_id)
    #
    # test_process_step.get_project_id()
    # print(test_process_step.project_id)
