import sys
#from util.configs import SalesOnboardingConfigs

import json
import datetime
from inspect import getsourcefile
from os.path import abspath

class User:

    def __init__(self, user_id=None, **kwargs):

        # Declare and/or assign the class properties
        if user_id is None:
            raise TypeError('A user ID must be provided to create an Albatross User object')

        elif kwargs.get('query_on_start', True):
            self.populate_user_information()

        else:
            self.user_id = user_id
            self.user_status = kwargs.get('user_status', '10')
            self.user_position = kwargs.get('user_position', None)
            self.user_start_date = kwargs.get('user_start_date', None)
            self.user_first_name = kwargs.get('user_first_name', None)
            self.user_last_name = kwargs.get('user_last_name', None)
            self.user_area = kwargs.get('user_area', None)
            self.user_area_id = kwargs.get('user_area_id', None)
            self.user_region = kwargs.get('user_region', None)
            self.user_region_id = kwargs.get('user_region_id', None)
            self.user_district = kwargs.get('user_district', None)
            self.user_district_id = kwargs.get('user_district_id', None)
            self.user_office = kwargs.get('user_office', None)
            self.user_office_id = kwargs.get('user_office_id', None)
            self.user_manager = kwargs.get('user_manager', None)
            self.user_recruiter = kwargs.get('user_recruiter', None)
            self.user_email = kwargs.get('user_email', None)
            self.user_recruiter_id = kwargs.get('user_recruiter_id', None)
            self.user_recruiter_email = kwargs.get('user_recruiter_email', None)
            self.user_recruiter_name = kwargs.get('user_recruiter_name', None)
            self.user_manager_first_name = kwargs.get('user_manager_first_name', None)
            self.user_manager_last_name = kwargs.get('user_manager_last_name', None)

    def create_property(self, property_name=None, property_value=None):

        if property_name not in self.__dict__.keys():
            self.__setattr__(self, property_name, property_value)

        else:
            raise AttributeError(f'{property_name} already exists - please update the property instead.')

    def update_property(self, property_name=None, property_value=None):

        if property_name in self.__dict__.keys():
            self.__setattr__(self, property_name, property_value)

        elif property_name in User.__dict__.keys():
            User.__dict__[property_name] = property_value

        else:
            self.create_property(self, property_name, property_value)

    def load_user_information(self, user_id=None):
        """
        This function will query the database to grab the required data for this User.
        This function does NOT update the AlbatrossUser properties, but loads the dictionary
        and returns it.
        """
        user_info_query_string = """
            WITH NEW_SALES_USERS AS (
                with recruited_by as (
                        select 	
                            ucfv.int_value "recruiter_user_id",
                            u.first_name || ' ' || u.last_name "recruiter_name",
                            ucfv.user_id "user_id"
                        from
                            flow.user_custom_field_value ucfv
                        left join flow.user_position up on
                            up.id = ucfv.int_value
                        left join flow.position pos on
                            pos.id = up.position_id
                        left join flow.custom_field_group_assignment cfga on
                            cfga.id = ucfv.custom_field_group_assignment_id
                        left join flow.custom_field cf on
                            cf.id = cfga.custom_field_id
                        left join flow.user u on u.id = ucfv.int_value
                        where 
                            cfga.id = 330
                            and 
                            cf.id = 455
                    )
                    select 
                        DISTINCT u.id,
                        u.first_name,
                        u.last_name,
                        ust.user_status_type,
                        pos.position,
                        up.start_date,
                        up.end_date,
                        (uphv.hierarchy::json -> 4)::json ->> 'orgName' "area",
                        (uphv.hierarchy::json -> 4)::json ->> 'orgId' "area_id",
                        (uphv.hierarchy::json -> 4)::json ->> 'level' "area_level",
                        (uphv.hierarchy::json -> 5)::json ->> 'orgName' "region", -- THIS is the manager
                        (uphv.hierarchy::json -> 5)::json ->> 'orgId' "region_id",
                        (uphv.hierarchy::json -> 5)::json ->> 'level' "region_level",
                        (uphv.hierarchy::json -> 6)::json ->> 'orgName' "district",
                        (uphv.hierarchy::json -> 6)::json ->> 'orgId' "district_id",
                        (uphv.hierarchy::json -> 6)::json ->> 'level' "district_level",
                        (uphv.hierarchy::json -> 7)::json ->> 'orgName' "office",
                        (uphv.hierarchy::json -> 7)::json ->> 'orgId' "office_id",
                        (uphv.hierarchy::json -> 7)::json ->> 'level' "office_level",
                        rb.recruiter_user_id "recruiter_user_id",
                        rb.recruiter_name "recruiter_name",
                        u.email "user_email",
                        u_rec.email "recruiter_email"

                    from flow.user u

                    inner join flow.user_position up on 
                        (up.user_id = u.id
                        and
                        up.primary_flag is True)

                    inner join flow.position pos on
                        pos.id = up.position_id

                    inner join flow.company com on
                        com.id = pos.company_id

                    inner join flow.org on
                        org.id = up.org_id

                    inner join flow.org_type ot on
                        ot.id = org.org_type_id

                    inner join flow.user_positions_vw upv
                        on upv.user_id = u.id

                    inner join flow.user_status_type ust
                        on ust.id = upv.user_status_type_id

                    inner join flow.user_position_hierarchy_vw uphv on
                        uphv.user_id = u.id

                    left join recruited_by rb on
                        rb.user_id = u.id

                    left join flow.user u_rec on
                        u_rec.id = rb.recruiter_user_id

                    where 
                        (""" + self.user_id_str + r""" = u.id)
                        or
                        (ust.user_status_type = 'New Request'
                        and
                        (
                            pos.position = 'Closer' 
                            or 
                            pos.position = 'Setter'
                        )
                        and
                        up.start_date is not null
                        and
                        up.end_date is null
                        and
                        up.archived is False
                        and
                        up.primary_flag is True)
            )
            Select 
                row_to_json(NEW_SALES_USERS.*) "json_info_new_sales_user"

            from 
                NEW_SALES_USERS
        """

        all_users_info_query_string = """
                    WITH NEW_SALES_USERS AS (
                        with recruited_by as (
                                select 	
                                    ucfv.int_value "recruiter_user_id",
                                    u.first_name || ' ' || u.last_name "recruiter_name",
                                    ucfv.user_id "user_id"
                                from
                                    flow.user_custom_field_value ucfv
                                left join flow.user_position up on
                                    up.id = ucfv.int_value
                                left join flow.position pos on
                                    pos.id = up.position_id
                                left join flow.custom_field_group_assignment cfga on
                                    cfga.id = ucfv.custom_field_group_assignment_id
                                left join flow.custom_field cf on
                                    cf.id = cfga.custom_field_id
                                left join flow.user u on u.id = ucfv.int_value
                                where 
                                    cfga.id = 330
                                    and 
                                    cf.id = 455
                            )
                            select 
                                DISTINCT u.id,
                                u.first_name,
                                u.last_name,
                                ust.user_status_type,
                                pos.position,
                                up.start_date,
                                up.end_date,
                                (uphv.hierarchy::json -> 4)::json ->> 'orgName' "area",
                                (uphv.hierarchy::json -> 4)::json ->> 'orgId' "area_id",
                                (uphv.hierarchy::json -> 4)::json ->> 'level' "area_level",
                                (uphv.hierarchy::json -> 5)::json ->> 'orgName' "region", -- THIS is the manager
                                (uphv.hierarchy::json -> 5)::json ->> 'orgId' "region_id",
                                (uphv.hierarchy::json -> 5)::json ->> 'level' "region_level",
                                (uphv.hierarchy::json -> 6)::json ->> 'orgName' "district",
                                (uphv.hierarchy::json -> 6)::json ->> 'orgId' "district_id",
                                (uphv.hierarchy::json -> 6)::json ->> 'level' "district_level",
                                (uphv.hierarchy::json -> 7)::json ->> 'orgName' "office",
                                (uphv.hierarchy::json -> 7)::json ->> 'orgId' "office_id",
                                (uphv.hierarchy::json -> 7)::json ->> 'level' "office_level",
                                rb.recruiter_user_id "recruiter_user_id",
                                rb.recruiter_name "recruiter_name",
                                u.email "user_email",
                                u_rec.email "recruiter_email"

                            from flow.user u

                            inner join flow.user_position up on 
                                (up.user_id = u.id
                                and
                                up.primary_flag is True)

                            inner join flow.position pos on
                                pos.id = up.position_id

                            inner join flow.company com on
                                com.id = pos.company_id

                            inner join flow.org on
                                org.id = up.org_id

                            inner join flow.org_type ot on
                                ot.id = org.org_type_id

                            inner join flow.user_positions_vw upv
                                on upv.user_id = u.id

                            inner join flow.user_status_type ust
                                on ust.id = upv.user_status_type_id

                            inner join flow.user_position_hierarchy_vw uphv on
                                uphv.user_id = u.id

                            left join recruited_by rb on
                                rb.user_id = u.id

                            left join flow.user u_rec on
                                u_rec.id = rb.recruiter_user_id

                            where                                 
                                (ust.user_status_type = 'New Request'
                                and
                                (
                                    pos.position = 'Closer' 
                                    or 
                                    pos.position = 'Setter'
                                )
                                and
                                up.start_date is not null
                                and
                                up.end_date is null
                                and
                                up.archived is False
                                and
                                up.primary_flag is True)
                    )
                    Select 
                        row_to_json(NEW_SALES_USERS.*) "json_info_new_sales_user"

                    from 
                        NEW_SALES_USERS
                """

        if user_id is None and self.user_id is None:
            raise TypeError('No value provided for the user ID')

        try:
            from BRSDBQ import post_query
        except ImportError:
            from .. import db_conn
        except Exception as e:
            Sys.exit(e)

        try:
            qry_response = post_query(query_string=user_info_query_string)
            print(len(qry_response))
            print(qry_response)
            return_list = []
            return_dict_list = []

            
        except Exception as e:
            try:
                qry_response = db_conn.post_query(query_string=user_info_query_string)
                print(len(qry_response))
                print(qry_response)
                return_list = []
                return_dict_list = []
                
            except:
                Sys.exit(e)
            

        for user_response in qry_response:
            return_list.append(json.dumps(user_response))

        for user_json_str in return_list:
            return_dict_list.append(json.loads(user_json_str)[0])

        return return_dict_list

    def populate_user_information(self, **kwargs):
        """
        This should either take a dictionary and fill in the AlbatrossUser object's properties,
        or this should query the DB to fetch the required info before writing to the object's properties
        """

        if kwargs is None:

            _loaded_info = self.load_user_information()
            if isinstance(_loaded_info, list):
                _loaded_info = _loaded_info[0]

        else:
            _loaded_info = kwargs

        #  Conditional because MANAGER_INFO isn't always present
        try:
            if not _loaded_info['manager_full_name'] is None:
                self.user_manager = _loaded_info['manager_full_name']
                self.user_manager_first_name = _loaded_info['manager_first_name']
                self.user_manager_last_name = _loaded_info['manager_last_name']
        except Exception as e:
            print(f'Error {e} occurred - Manager Information Missing')
            pass

        #  Conditional because RECRUITER_INFO isn't always present
        try:
            if not _loaded_info['recruiter_name'] is None:
                self.user_recruiter_id = _loaded_info['recruiter_user_id']
                self.user_recruiter_email = _loaded_info['recruiter_email']
                self.user_recruiter_name = _loaded_info['recruiter_name']
        except Exception as e:
            print(f'Error {e} occurred when attempting to load user info - Recruiter info missing')
            pass

        #  Set all other User Properties
        for (key, pair) in _loaded_info:
            self.update_property(self, key, pair)

        return _loaded_info

    def get_all_properties(self):
        """
        WIP - Maybe want to have this as the string function and make another function to
        return dictionary of properties for the object
        """

        print(f'User Name: {self.user_first_name} {self.user_last_name}')
        print(f'User ID: {self.user_id_str}')
        print(f'User Position: {self.user_position}')
        print(f'User Status: {self.user_status}')
        print(f'User Email: {self.user_email}')
        print(f'User Start Date: {self.user_start_date}')
        print(f'User Area: {self.user_area}')
        print(f'User Region: {self.user_region}')
        print(f'User District: {self.user_district}')
        print(f'User Office: {self.user_email}')

    def update_user_status(self,
                           user_status=None,
                           open_session=None):
        """
        This function will set the provided user ID to pending if no user status is provided,
        or to the provided status
        """

        status_type = {
            '10': 'Pending',
            '9': 'Active',
            '139': 'Active',
            '113': 'Active',
            '65': 'Active',
            '13': 'Expired',
            '11': 'Terminated',
            '16': 'Leave of Absence',
            '129': 'Active',
            '15': 'Inactive',
            '137': 'Active',
            '14': 'Pending Termination',
            '12': 'New Request',
            '121': 'Active'
        }

        status_type_id = {
            'Pending': '10',
            'Active': '9',
            'Expired': '13',
            'Terminated': '11',
            'Leave of Absence': '16',
            'Inactive': '15',
            'Pending Termination': '14',
            'New Request': '12'
        }

        if user_status is None:
            _user_status = status_type['10']

        elif isinstance(user_status, int):  # If Int Status provided
            _user_status = status_type[str(user_status)]

        elif isinstance(user_status, str):  # If str status provided

            if user_status in status_type.keys():  # If string key
                _user_status = status_type[user_status]  # If string name
            else:
                _user_status = user_status

        else:
            _user_status = user_status

        try:
            from BRSoptions import alb_configs
            bearer_token_string = alb_configs.albatross_non_expiring_token
            _user_id = self.user_id

            if open_session is None:
                import requests
                _session = requests.Session()
                _session.headers.update({'Authorization': f'Bearer {bearer_token_string}'})
            else:
                _session = open_session

        except Exception as e:
            print('The following error occurred when attempting to initialize variables for updating the User Status'
                  f' in Albatross: {e}')
            return False  # Error code for first try block

        try:
            _session_response = _session.post(
                f'https://albatross-api.myblueraven.com/api/v1/flow/'
                f'user/{str(_user_id)}/'
                f'status/{status_type_id[_user_status]}',
                headers={'Content-Type': 'application/json'},
            )
            print(f'User status updated: {_session_response}')
            self.user_status = _user_status
        except Exception as e:
            print(
                f'While attempting to update the User Status custom field, the following error occurred: {e}')
            return False

        return True

