# BRSComms Module
# Created by Christopher Bahr 2021
import sys
import os

from . import sms
from . import file_share
from . import email

from brs_tk.util.data import BRSValues
from brs_tk.util.data import Auth

from .email.email import get_code, api_endpoint, support_device_flow_session, profile_photo, \
    send_mail, sharing_link, upload_file, send_mail_from_support_shared_inbox, \
    support_shared_inbox_device_flow_session, final_design_inbox_flow_session

import re
import urllib
import time
import os.path
import pprint
import shutil
import requests
import json
import datetime
from inspect import getsourcefile
from os.path import abspath

# # # SESSION FUNCTIONS # # # #
def open_session_support_inbox(arg_client_id,
                               auto=False):

    try:
        return_session = support_device_flow_session(arg_client_id)
        return return_session
    except Exception as e:
        print(f'ERROR {e}')
        return False


def open_session_support_shared_inbox(arg_client_id,
                                      auto=False):
    try:
        return_session = support_shared_inbox_device_flow_session(arg_client_id)
        return return_session
    except Exception as e:
        print(f'ERROR {e}')
        return False

def open_session_support_shared_inbox(arg_client_id,
                                      auto=False):
    try:
        return_session = support_shared_inbox_device_flow_session(arg_client_id)
        return return_session
    except Exception as e:
        print(f'ERROR {e}')
        return False

# # # # SEND EMAIL FUNCTIONS # # # # #
def send_fd_email(session,
                  customer_info,
                  test_mode_active=False,
                  **kwargs):
    """
    Send email from authenticated user.

    session = requests.Session() instance with a valid access token for
    Microsoft Graph in its default HTTP headers

    :param session:
    :param customer_info:
    :return:
    """

    print('Now sending Final Design email . . .')

    if kwargs.get('config', False) == False:
        from brs_tk.util import config
        _configs = config.ConfigSetup(config_type='finch')
    else:
        _configs = kwargs.get('config', False)

    # if test_mode_active is True:

    # Initialize recipient constants
    if _configs.bcc_sierra is True:
        _sierra_recipient = {
            "emailAddress": "sierra.cobb@blueravensolar.com",
            "name": "Sierra Cobb"
        }

    if _configs.bcc_chris is True:
        _chris_recipient = {
            "emailAddress": "christopher.bahr@blueravensolar.com",
            "name": "Chris Bahr"
        }

    _customer_recipient = {
        "emailAddress": str(customer_info["contact_email"]),
        "name": str(customer_info["project_name"])
    }

    _closer_recipient = {
        "emailAddress": str(customer_info["closer_email"]),
        "name": str(customer_info["closer_email"])
    }

    print('making recipient list')
    # Create lists of the recipients
    _bccRecipients = []
    if _configs.bcc_sierra is True:
        _bccRecipients.append(_sierra_recipient)
    if _configs.bcc_chris is True:
        _bccRecipients.append(_chris_recipient)

    _ccRecipients = [
        _closer_recipient
    ]

    _toRecipients = [
        _customer_recipient
    ]

    sent_from = {
        "emailAddress": {
            # "emailAddress": "support@blueravensolar.com",
            "emailAddress": "finaldesign_donotreply@blueravensolar.com",
            "name": "Blue Raven Solar Customer Support"
        }
    }
    # Assign the recipient lists for a recipient list type dictionary, to pass to the send mail function

    _recipients = {
        "to_recipients": _toRecipients,
        "cc_recipients": _ccRecipients,
        "bcc_recipients": _bccRecipients
    }

    print('getting endpoint')
    try:
        user_profile = session.get(api_endpoint('me'))
        if not user_profile.ok:
            pprint.pprint(user_profile.json())  # display error
            return 0
    except:
        print('oof - returning -1')
        return -1

    try:
        user_data = user_profile.json()
        email = user_data['mail']
        display_name = user_data['displayName']
    except Exception as e:
        print(f'While grabbing email API data, the following error occurred: {e}')
        return -2

    # Choose inputs based on if this is testing mode or not
    # if testing_mode:
    #     email_to = "christopher.bahr@blueravensolar.com"
    if test_mode_active is True:

        pass
    else:
        print('changing directory for some reason, for template maybe?')
        try:

            # print('Send mail ---------------->',
            #      'https://graph.microsoft.com/beta/me/microsoft.graph.sendMail')
            # link_url = "https://images.app.goo.gl/D58SdHdovQ8QWHFt9"

            # Change current directory to the location of this Python code
            os.chdir(_configs.default_download_location)
        except Exception as e:
            print(f'While attempting to navigate to the script\'s file directory, the following error occurred: {e}')
            return -3

        try:
            print(f'configs fd email tempalte filepath is {_configs.fd_email_template_filepath}')
            with open(_configs.fd_email_template_filepath) as template_file:
                template = template_file.read().format(name=customer_info["project_name"])
        except Exception as e:
            print(f'While attempting to fill out the Final Design email template, the following error occurred: {e}')
            return -4

        try:
            final_design_name = customer_info['file_name']
            timeout = 0
            print('trying to move final design file')
            print(f'final_design_name = {final_design_name}')
            print(f'customer info final design download path: {(customer_info["final_design_download_path"])}')
            while timeout < 15 and os.path.exists(os.path.abspath(os.path.dirname(sys.argv[0]))
                                                  + '\\' + final_design_name) is True:
                # Try moving Final Design file into self-directory if it is not already there. 15 seconds to try
                try:
                    dest = shutil.move(customer_info['final_design_download_path'],
                                       os.path.abspath(os.path.dirname(sys.argv[0]))
                                       + '\\' + customer_info['file_name'])
                    print('weird af file path')
                    print(os.path.abspath(os.path.dirname(sys.argv[0]))
                                       + '\\' + customer_info['file_name'])
                    break

                except:
                    import time
                    time.sleep(1)
                    timeout += 1
        except Exception as e:
            print(f'While attempting to move the Final Design PDF into the python file directory, the following error'
                  f' occurred: {e}')
            return -5
    if test_mode_active is True:
        try:
            send_response = send_mail(session=session,
                                      subject='Final Design Complete',
                                      recipients=_recipients,
                                      body='TESTING',
                                      attachments=[customer_info['final_design_download_path']],
                                      attachment_short_names=[final_design_name]
                                      )
        except Exception as e:
            print(f'While attempting to send the Final Design Email and/or parse the response, the following error'
                  f' occurred: {e}')
            import time
            # time.sleep(45)
            return -6
    else:
        try:
            print('about to actually attempt to send email')
            print(f'recipient: {_recipients}')
            send_response = send_mail(session=session,
                                      subject='Final Design Complete',
                                      recipients=_recipients,
                                      body=template,
                                      attachments=[customer_info['final_design_download_path']],
                                      attachment_short_names=[final_design_name]
                                      )
            print('send_response has run!')
            print(send_response.text)
        except Exception as e:
            print(f'While attempting to send the Final Design Email and/or parse the response, the following error'
                  f' occurred: {e}')
            import time
            # time.sleep(45)
            return -6

    try:
        # print(28 * ' ' + f'<Response [{send_response.status_code}]>')
        if not send_response.ok:
            pprint.pprint(send_response.json())  # show error message
            import time
            return -2
            # time.sleep(45)
    except Exception as e:
        print(f'While attempting to  parse the response to sending the Final Design email, the following error'
              f' occurred: {e}')
        import time
        # time.sleep(45)
        return -7

    # Delete Final Design after sending
    if test_mode_active is True:
        return 0
        pass
    else:
        print('now attempting to remove FD file')
        try:
            os.remove(os.path.abspath(os.path.dirname(sys.argv[0])) + '\\' + customer_info['file_name'])
            print('file deleted, returning 2')
            return 2
        except Exception as e:
            try:
                os.remove(customer_info['final_design_download_path'])
                return 100
            except:
                print(
                    f'While attempting to delete the Final Design that was just emailed, the following error occurred: {e}')
                return 1