#####################################################
#   Aurora API Module
#   Created by Chris Bahr
#   For Blue Raven Solar
#   4/16/2021
#   bahrbarian@gmail.com
#####################################################


# Import required modules
import sys
import os
import urllib.request as ur
import urllib.parse
import datetime
import base64, hashlib, hmac
import json

# Declare class for a specific aurora design
class auroraDesign:

    def __init__(self, auroraDesignID, **kwargs):
        """ Create a new new instance of an Aurora Design ID, with data loaded into Python from the API call"""
        self.auroraDesignID = auroraDesignID
        designID = auroraDesignID


        #################### DECLARE AND ASSIGN VARIABLES

        # Production Values ----- You may need to update the values below to match the tenant/account for the Aurora Design you are referencing
        prodHost = 'https://api.aurorasolar.com/v2/tenants/' # API Endpoint
        prodTenant = '8ca5251c-ab3c-41d0-b159-4f51ff3eb67a' # Tenant ID
        prodKey = '9b26d6a5-ed5f-40cc-9621-a92a8d94ec47' # API Key
        prodSecret = '6efe7e0d-03b4-419a-8c5f-cf5665cd13b0' # API Secret

        # Declaring variables for code below
        host = prodHost
        tenantID = prodTenant
        apiKey = prodKey
        apiSecret = prodSecret

        #################### ASSIGN REQUEST PARAMETERS

        # Declare and Format Time Stamp Variables
        timestamp = str(datetime.datetime.utcnow())
        urltimestamp = timestamp[0:19] + ' UTC'
        formattedTime = urltimestamp.replace(' ', '%20')

        # Target Design ID to Fetch Summary
        designID = auroraDesignID

        # Set Full Request URL
        queryRequestLong = host + tenantID + '/designs/' + designID + '/summary'

        # Request URL Without the full host, per suggestion from Aurora
        queryRequestShort = '/v2/tenants/' + tenantID + '/designs/' + designID + '/summary'

        # URL to pass to hash
        stringToSign = 'GET\n' + queryRequestShort + '\nAuroraKey=' + apiKey + '\nTimestamp=' + formattedTime + '\n'

        secretBytes = bytes(apiSecret, "utf-8")
        rawString = bytes(stringToSign, "utf-8")
        hashed = hmac.new(secretBytes, rawString, hashlib.sha256)
        signed = base64.b64encode(hashed.digest()).decode()

        ####################  MAKE THE REQUEST

        # Declare Headers
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

        # Declare Request Object
        fullQueryRequest = queryRequestLong + '?AuroraKey=' + apiKey + '&Timestamp=' + formattedTime + '&Signature=' + signed

        # Make Request to URL
        request = ur.Request(fullQueryRequest, headers=headers)

        # Parse Request Response
        response_body = json.load(ur.urlopen(request))

        # Start filtering through summary down to desired design attributes
        designSummary = response_body['design']
        #print(designSummary)
        #print(designSummary['arrays'])
        productionEstimate = designSummary['energy_production']
        monthlyEstimates = productionEstimate['monthly']

        # Create Dict for Production Estimate by Month
        monthlyProdDic = {'January': monthlyEstimates[0], 'February': monthlyEstimates[1], 'March': monthlyEstimates[2],
                          'April': monthlyEstimates[3],
                          'May': monthlyEstimates[4], 'June': monthlyEstimates[5], 'July': monthlyEstimates[6],
                          'August': monthlyEstimates[7], 'September': monthlyEstimates[8],
                          'October': monthlyEstimates[9], 'November': monthlyEstimates[10],
                          'December': monthlyEstimates[11]
                          }

        self.monthlyProductionEstimate = monthlyProdDic
        self.yearlyProductionEstimate = productionEstimate['annual']

        # Declare array/module variables
        # TODO - Create way to do this for multiple arrays
        self.arrayQuantity = len(designSummary['arrays'])
        self.arrays = designSummary['arrays']
        self.moduleBrand = designSummary['arrays'][0]['module']['manufacturer']
        self.moduleWattage = designSummary['arrays'][0]['module']['rating_stc']
        self.orientation = designSummary['arrays'][0]['module']['orientation']


        # Find total number of panels
        self.panelTotal = 0
        currentArray = 0
        while currentArray < self.arrayQuantity:
            self.panelTotal += designSummary['arrays'][currentArray]['module']['count']
            currentArray +=1

    # Summarize Aurora Design Details if class name called
    def __str__(self):
        response = "Aurora Design ID - "+self.auroraDesignID+"\n"
        response += 'Panel Total - '+str(self.panelTotal) + '\n'
        response += 'Array Quantity - ' + str(self.arrayQuantity) + '\n'
        response += 'Module Brand - ' + str(self.moduleBrand) + '\n'
        response += 'Module Wattage - ' + str(self.moduleWattage) + '\n'
        response +='Yearly Production Estimate - '+str(self.yearlyProductionEstimate) + '\n'
        response +='Monthly Production Estimates: \n'+str(self.monthlyProductionEstimate) + '\n'
        return response


    def getMonthlyProductionEstimates(self):
        """Returns the montlhy production estimate for the entire design"""
        return self.monthlyProductionEstimate

    def getArrayQuantity(self):
        """Returns the number of arrays in the design"""
        return self.arrayQuantity

    def getModuleBrand(self):
        """Returns the brand of the first array's modules"""
        return self.moduleBrand

    def getModuleWattage(self):
        """Returns the wattage of the first array's modules"""
        return self.moduleWattage

    def getOrientation(self):
        """Returns the orientation of the first array's modules"""
        return self.orientation

    def getTotalPanels(self):
        """Returns the total number of panels in the design"""
        return self.panelTotal

    def getYearlyProductionEstimate(self):
        """Returns yearly production estimate for design"""
        return self.yearlyProductionEstimate

