# BRS Financiers Module

import sys
import os

try:
    from ...util.data import Auth
    from ...util.data import FinchValues
    from ...util.data import ThirdPartyValues
    from ...util.data import BRSValues
    # from BRSoptions import alb_configs
    # from BRSoptions import finch_configs
    # from BRSoptions import finch_test_vals
    # from BRSoptions import mail_configs
    # from BRSoptions import mail_configs_alt
    # from BRSoptions import external_configs
    #
    # fdo = finch_configs
    # ftest = finch_test_vals
    # mail_config_alt = mail_configs_alt
    # mail_config = mail_configs
    # br_options = alb_configs
except Exception as br_e:
    print(f'Due to the following error, BRS modules were not imported correctly - {br_e}')

    ### MODULE IMPORT SECTION ###
try:

    import re
    import urllib
    import time
    import os.path
    import pprint
    import shutil

    try:
        import requests
    except:
        os.system('pip install requests')
        import requests

    try:
        from selenium import webdriver
    except:
        os.system('pip install selenium')
        from selenium import webdriver

    from selenium.webdriver.common.keys import Keys
    from selenium.webdriver.chrome.options import Options
    from selenium.webdriver.common.action_chains import ActionChains

    import json

    import datetime
    from inspect import getsourcefile
    from os.path import abspath

except Exception as e:
    print(f'{e} --- There was an error in importing the initial modules')
    sys.exit()

    ### VARIABLES ###
try:
    ### GLOBAL VARIABLES ###
    windowless_chrome_mode = True
    albatross_bearer_token = Auth.albatross_bearer_token



except:
    pass


# TODO - Errors in functions should still return dictionaries detailing progress so far, the driver used, etc
def format_date_string_for_albatross(date_text,
                                     **kwargs):
    """
    Accepts a string of a date as grabbed from Loanpal, and formats it to be passed to Albatross API

    -Returns formatted date as a string if successful.
    -Returns -1 if there is an error

    :param date_text:
    :return:
    """
    print('Formatting date string for Albatross . . .')
    if 'test_mode' in kwargs:
        _test_mode_active = kwargs['test_mode']
    else:
        _test_mode_active = False

    try:
        _date_input = date_text
        _date_with_dashes = str.replace(_date_input, '/', '-')
        _date_month = _date_with_dashes[0:2]
        _date_day = _date_with_dashes[3:5]
        _date_year = _date_with_dashes[6:8]

        _formatted_date = '20' + _date_year + '-' + _date_month + '-' + _date_day + 'T00:00:00Z'

        # print(_formatted_date)
        return _formatted_date

    except:
        return -1


def check_financier_portal(customer_info,
                           loan_product_info,
                           test_mode_active=False):
    """Accepts a dictionary of customer_info as pulled from the Albatross query, and a dictionary of structured loan
    product information.

    First, the loan product information is checked against a list of financier/product combinations currently supported
    by this function.
        - The function returns 0 if it is not in this list.

    With the provided loan product information, the following actions are taken if possible:
        - Checking the financier portal for any updates
        - Making any possible updates in the Financier portal like sending loan docs or verifying loan details
        - Makes any possible updates to Albatross custom fields (like updating for Loan Docs Sent date)
        - For cash projects, this will just mark the custom fields and start any downpayment process if necessary

    On successful completion:
        -Function should return the dates for the different loan actions, actions taken, etc. to
        allow the parent function to then record/update as necessary.

    On error:
        - Function should return a negative number corresponding to the location of the error.

    :param test_mode_active:
    :param customer_info:
    :param loan_product_info:
    :return:
    """

    print('Now checking in the financier portal if supported . . .')

    # Set Chrome Driver Options
    chromeOptions = Options()
    if windowless_chrome_mode is True:
        chromeOptions.add_argument("--headless")
    chromeOptions.add_argument("--window-size=1920x1080")
    chromeOptions.add_argument("--start-maximized")
    chromeOptions.add_experimental_option("prefs", {
        "download.default_directory": fddddddddd65jkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkko.fd_download_location,
        "download.prompt_for_download": False,
        "download.directory_upgrade": True,
        "safebrowsing_for_trusted_sources_enabled": False,
        "safebrowsing.enabled": False
    })

    # Initialize Login Information
    _goodleap_login_url = ThirdPartyValues.GoodLeap.goodleap_login_url
    _goodleap_loan_search_url = ThirdPartyValues.GoodLeap.goodleap_loan_search_url
    _goodleap_password = ThirdPartyValues.GoodLeap.goodleap_password
    _goodleap_login = ThirdPartyValues.GoodLeap.goodleap_login

    # Create list of supported loan products
    _supported_loan_products = [
        '_loanpal_25yr_199',
        '_loanpal_25yr_299',
        '_loanpal_25yr_198',
        '_loanpal_20yr_149',
        '_loanpal_15yr_499',
        '_loanpal_12yr_299',
        '_loanpal_10yr_499',
        '_loanpal_7yr_699',
        '_loanpal_25yr_199',
        '_cash'
    ]

    # Check loan product dictionary against accepted loan products
    try:
        if loan_product_info['loan_product_name'] not in _supported_loan_products:
            print(loan_product_info['loan_product_name'] + ' was not found in the list of supported loan products.')
            print('The following are supported loan products: ' +
                  ''.join(str(entry) for entry in _supported_loan_products))

            return 0  # Return -1 if the loan product is not a supported type
    except Exception as e:
        print(f'While attempting to check the loan product dictionary against the supported loan products, the '
              f'following error occurred: {e}')
        # TODO - LOG ERROR
        _default_dic = {
            'boolean_loan_approved': False,
            'loan_approved_date': False,
            'boolean_loan_documents_sent': False,
            'loan_documents_sent_date': False,
            'loan_documents_sent_by': False,
            'boolean_loan_documents_signed': False,
            'loan_documents_signed_date': False,
            'boolean_loan_details_verified': False,
            'loan_details_verified_date': False,
            'loan_details_verified_by': False,
            'boolean_has_stips': False
        }

        return -1

    ### FOR CASH PROJECTS ###
    try:
        if (loan_product_info['financier'] == 'Cash'
                or loan_product_info['financier'] == 'Customer'):

            print('CASH PROJECT IDENTIFIED')

            _date_today = datetime.datetime.today()
            # print(_date_today)
            _pre_formatted_date = _date_today.strftime('%m-%d-%y')
            # print(_pre_formatted_date)

            # Outcome Dictionary for Cash Projects
            _financier_portal_outcome_info = {
                'boolean_loan_approved': True,
                'loan_approved_date': _pre_formatted_date,
                'boolean_loan_documents_sent': True,
                'loan_documents_sent_date': _pre_formatted_date,
                'loan_documents_sent_by': 'Support',
                'boolean_loan_documents_signed': True,
                'loan_documents_signed_date': _pre_formatted_date,
                'boolean_loan_details_verified': True,
                'loan_details_verified_date': _pre_formatted_date,
                'loan_details_verified_by': 'Support',
                'boolean_has_stips': False,
                'errors': None
            }

            return _financier_portal_outcome_info

        # Outcome Dictionary for Loan Projects
        else:
            _financier_portal_outcome_info = {
                'boolean_loan_approved': False,
                'loan_approved_date': False,
                'boolean_loan_documents_sent': False,
                'loan_documents_sent_date': False,
                'loan_documents_sent_by': False,
                'boolean_loan_documents_signed': False,
                'loan_documents_signed_date': False,
                'boolean_loan_details_verified': False,
                'loan_details_verified_date': False,
                'loan_details_verified_by': False,
                'boolean_has_stips': False,
                'errors': None
            }
    except Exception as e:
        print(f'While attempting to initialize the financier portal outcome dictionaries for the project, the following'
              f'error occurred: {e}')
        return -1.5

    ### FOR LOAN PROJECTS ###
    ###  Take actions and scrape data in financier portal  ##
    ## Log in to Financier Portal

    if loan_product_info['financier'] == 'LoanPal':
        ### Initialize headless Chrome driver
        driver = webdriver.Chrome(
            executable_path=BRSValues.DriverConfigs.chrome_driver_path,
            options=chromeOptions)

        ### Login to GoodLeap - Attempting to go straight to loan search page
        try:
            time.sleep(4)
            driver.get(_goodleap_loan_search_url)
            time.sleep(8)
            loginfield = driver.find_element_by_name('username')
            loginfield.send_keys(_goodleap_login)
            passwordfield = driver.find_element_by_id('password')
            passwordfield.send_keys(_goodleap_password)
            passwordfield.send_keys(Keys.RETURN)
            time.sleep(10)
        except Exception as e:
            print(f'While attempting to login to GoodLeap, the following error occurred: {e}')
            # TODO - LOG ERROR
            return -2

        ### Search for Customer Loan
        try:

            search_box = driver.find_elements_by_id('navSearchTextbox')
            search_box[0].send_keys(customer_info['project_id'])
            search_box[0].send_keys(Keys.RETURN)
            time.sleep(9)
            loan = driver.find_element_by_xpath('//*[@id="loans-table"]/tbody/tr[1]/td[1]/a').click()
            time.sleep(8)
        except Exception as e:
            print(f'While attempting to locate the customer\'s loan in GoodLeap, the following error occurred: {e}')
            # TODO - LOG ERROR
            return -3

        ### Scrape for dates/approvals of the various stages
        print('Loan found, now parsing data')
        try:
            time.sleep(3)

            # Loan Approval
            _loan_approved_status = driver.find_element_by_id('two')
            if _loan_approved_status.get_attribute("class") == "dot bg-success":
                _financier_portal_outcome_info['boolean_loan_approved'] = True
                _financier_portal_outcome_info['loan_approved_date'] = driver.find_element_by_css_selector(
                    '#two > div.timeline-date').text
            else:
                try:
                    _loan_stips_status = driver.find_element_by_xpath(f'//*[@id="stips"]')
                    if _loan_stips_status.get_attribute("class") == "dot bg-warning pointer clickable":
                        _financier_portal_outcome_info['boolean_has_stips'] = True
                except Exception as e:
                    print(f'While checking for stips since the loan isn\'t approved, the following error occurred: {e}')

            # Loan Documents Sent
            _loan_documents_sent_status = driver.find_element_by_id('three')
            if _loan_documents_sent_status.get_attribute("class") == "dot bg-success":
                _financier_portal_outcome_info['boolean_loan_documents_sent'] = True
                _financier_portal_outcome_info['loan_documents_sent_date'] = driver.find_element_by_css_selector(
                    '#three > div.timeline-date').text
                _financier_portal_outcome_info['loan_documents_sent_by'] = 'Support'

            # Loan Documents Signed
            _loan_documents_signed_status = driver.find_element_by_id('four')
            if _loan_documents_signed_status.get_attribute("class") == "dot bg-success":
                _financier_portal_outcome_info['boolean_loan_documents_signed'] = True
                _financier_portal_outcome_info['loan_documents_signed_date'] = driver.find_element_by_css_selector(
                    '#four > div.timeline-date').text

            # Loan Details Verified

            # More complicated check for loan details verified
            # _loan_details_verified_status = driver.find_element_by_xpath('//*[@id="six" and contains(text(),]/span[1]')

            try:
                _loan_details_verified_date = driver.find_element_by_css_selector(
                    '#six > div.timeline-date').text
                # print(_loan_details_verified_date + ' is the loan details verified date as '
                # 'fetched from the financier portal')
                if len(_loan_details_verified_date) > 1:
                    # print('Looks like the date was long enough to be real!')
                    _financier_portal_outcome_info['loan_details_verified_date'] = driver.find_element_by_css_selector(
                        '#six > div.timeline-date').text
                    _financier_portal_outcome_info['boolean_loan_details_verified'] = True
                    _financier_portal_outcome_info['loan_details_verified_by'] = 'Support'
            except:
                # print('The date length was too short to be a real date')
                pass  # If we couldn't grab the date, it means the loan details were not verified


        except Exception as e:
            print(f'While attempting to scrape the pertinent information from the loan page,'
                  f' the following error occurred: {e}')
            # TODO - LOG ERROR
            return -4

        ### Send loan docs if necessary
        try:
            # print("Loan docs sent yet? " + str(_financier_portal_outcome_info['boolean_loan_documents_sent']))
            if _financier_portal_outcome_info['boolean_loan_documents_sent'] is False \
                    and _financier_portal_outcome_info['boolean_loan_approved'] is True:
                _date_today = datetime.datetime.today()
                _pre_formatted_date = _date_today.strftime('%m-%d-%y')

                driver.find_element_by_id("sendDocsButton").click()
                _financier_portal_outcome_info['boolean_loan_documents_sent'] = True
                _financier_portal_outcome_info['loan_documents_sent_date'] = _pre_formatted_date
                _financier_portal_outcome_info['loan_documents_sent_by'] = 'Automation User'
                time.sleep(1)
                print('Just sent the Loan Docs')



        except Exception as e:
            print(f'While attempting to send the loan docs, the following error occurred: {e}')
            # TODO - LOG ERROR
            return -5

        ### Check System Price in GoodLeap
        if _financier_portal_outcome_info['boolean_loan_approved'] is True \
                and _financier_portal_outcome_info['boolean_loan_details_verified'] is False:
            try:
                _loanpal_system_price = driver.find_element_by_xpath(
                    '//*[@id="content"]/div[1]/div[3]/div/div[1]/dl/dd[2]').text

                # print('Found the loanpal system price as ' + _loanpal_system_price)
            except Exception as e:
                print(f'While attempting to check the System Price in GoodLeap, the following error occurred: {e}')
                # TODO - LOG ERROR
                return -6

            # Open prompt to Verify Loan Details if possible
            time.sleep(4)
            try:
                driver.find_element_by_xpath('//*[@id="six"]/span[2]/i').click()
            except Exception as e:
                try:
                    driver.find_element_by_xpath('//*[@id="six"]/span[1]/i').click()
                except Exception as e:
                    print(f'While attempting to check click the Verify Loan Details option in GoodLeap, '
                          f'the following error occurred: {e}')
                    # TODO - LOG ERROR
                    return -7

            # Open Loan Term dropdown
            try:
                time.sleep(1)
                _loan_term_dropdown = driver.find_elements_by_class_name("css-f3cxg0")
                _loan_term_dropdown[-1].click()
                time.sleep(1)
            except Exception as e:
                print(f'While attempting to find the loan term dropdown, the following error occurred: {e}')
                print('This is most likely because the IA hasn\'t been uploaded yet.')
                # TODO - LOG ERROR
                return [-8, _financier_portal_outcome_info]

            # Select Loan Term
            try:
                _loan_term_to_select = loan_product_info['dropdown_name']
                time.sleep(3)
                _select_array = driver.find_elements_by_xpath(
                    f'//li[starts-with(@id, "downshift-") and contains(text(),"{_loan_term_to_select}")]')

                # Attempt to select the correct loan term dropdown option
                for element in _select_array:

                    if _loan_term_to_select in element.text:  # We only want to click the dropdown element

                        try:
                            time.sleep(1.5)
                            ActionChains(driver).send_keys(Keys.PAGE_DOWN).perform()
                            element.click()
                            break
                        except Exception as e:
                            try:
                                ActionChains(driver).send_keys(Keys.ARROW_UP).perform()
                                ActionChains(driver).send_keys(Keys.ARROW_UP).perform()
                                ActionChains(driver).send_keys(Keys.ARROW_UP).perform()
                                time.sleep(2)

                                # print('Trying to click element')
                                element.click()
                                break
                            except:
                                print(
                                    f'While attempting to click the Loan Term dropdown option, the following '
                                    f'error occurred: {e}')
                                # TODO - LOG ERROR
                                return -8.5

                            print(f'While attempting to click the Loan Term dropdown option, the following '
                                  f'error occurred: {e}')
                            # TODO - LOG ERROR
                            return -9
            except Exception as e:
                print(f'While attempting to find the loan term dropdown, the following error occurred: {e}')
                # TODO - LOG ERROR
                return -10

            # Enter Loan Price
            try:
                _project_details_loan_amount = str(customer_info['loan_amount'])
                _loan_price_field = driver.find_elements_by_class_name("input-prefix")
                _loan_price_field[-1].send_keys(_loanpal_system_price)
                if testing_mode is True:
                    print('Pausing here for testing to keep from anything being submitted')
                    time.sleep(5)
                else:
                    _loan_price_field[-1].send_keys(Keys.RETURN)
            except Exception as e:
                print(f'While attempting to input the Loan Amount for Verifying Loan Details, the following '
                      f'error occurred: {e}')
                # TODO - LOG ERROR
                return -11

            # Click Submit button for Loan Detail Verification
            try:
                _loan_details_submit_button = driver.find_element_by_xpath(
                    f'//*[@type="button" and @data-component="Button" and contains(text(),"Submit")]')
                _verify_details_submit_outcome = _loan_details_submit_button.click()
                # print(str(_verify_details_submit_outcome))
                time.sleep(3)
            except Exception as e:

                # print('Attempting to scroll to element first')
                try:
                    time.sleep(3)
                    _loan_details_submit_button = driver.find_element_by_xpath(
                        f'//*[@type="button" and @data-component="Button" and contains(text(),"Submit")]')

                    action = ActionChains(driver)
                    action.move_to_element(_loan_details_submit_button).perform()
                    time.sleep(3)
                    _verify_details_submit_outcome = action.click().perform()

                    time.sleep(3)

                except Exception as e2:
                    print(f'This error occurred on the first attempt to click the Submit element: {e}')
                    print(f'During the second attempt to click the Submit element, the following error occurred: {e2}')
                    return -11.5

            # Update Financier Portal Outcome Info dictionary
            try:
                _date_today = datetime.datetime.today()
                _pre_formatted_date = _date_today.strftime('%m-%d-%y')

                _financier_portal_outcome_info['boolean_loan_details_verified'] = True
                _financier_portal_outcome_info['loan_details_verified_date'] = _pre_formatted_date
                _financier_portal_outcome_info['loan_details_verified_by'] = 'Automation User'
            except Exception as e:
                print(f'While attempting to update the financier portal outcome dictionary, the following error'
                      f'occurred: {e}')
                # TODO - LOG ERROR
                return -12

        ### After checking and updating as much as possible, return dictionary with details on statuses and actions taken
        try:
            return _financier_portal_outcome_info
        except Exception as e:
            print(f'While attempting to return the financier portal outcome info dictionary, the following error'
                  f'occurred: {e}')
            # TODO - LOG ERROR
            return -13


def get_structured_loan_data(customer_info):
    """This function determines and structures the financier, loan product, and loan term from the provided int values
    from the Albatross project details. There is a dictionary for converting the int_values of these custom fields
    to the corresponding text value. (This requires some maintenance as new options are added to Albatross)

    On successful completion, function should return the a dictionary with the pertinent loan product details

    On error, function should return a negative number corresponding to the location of the error.

    :param customer_info:
    :return:
    """

    print('Structuring loan data before passing to financier portal function . . .')

    # The next commented section is the information on int_values and corresponding names from the flow.list_of_value
    # table in Albatross
    '''
    Last updated 7/19/2021

    The following is a list of the integer values and corresponding values in Albatross
    for the int_values of the following custom fields.

   ** Products **
   Custom Field int values for project_details.product
   product = [
   293: BluePower Plus
   294: BluePower
   295: BluePower Plus Prepaid
   296: Blue Power Save+
   8984: BluePower
   8986: BluePower
   8987: BluePower
   8988: BluePower
   8990: BluePower
   8992: BluePower
   8993: BluePower
   8971: BluePower Plus
   19424: SmartStart
   ]

    ** Loan Terms **
    Custom Field int values for project_details.loan_term
    loan_term = [    

    <project_details.loan_term int value> : <Loan Term Length in Years>    
    19110 : 7 
    424 : 10
    425 : 12
    426 : 15
    427 : 20
    428 : 25    

    ]


    ** GoodLeap ** (formerly known as LoanPal)
    *
    ---Custom Field int values for project_details.financier
    primary_financier = [
    116, 334, 722, 731, 7538, 7539, 7540, 7541, 7542, 7543, 7544, 7545, 7546, 7547, 7548, 7549, 7550, 9410, 9411, 9412,
    9413, 9414, 9415, 9416, 9417, 9418, 9419, 9420, 9421, 9422, 13219, 13220, 13221, 13222, 13223,13224, 13225, 13226,
    13227, 13228, 13229, 13230, 13231, 13310, 13311, 13312, 13313, 13314, 13315, 13316, 13317, 13318, 13319, 13320,
    13321, 13322
    ] 




    ** Mosaic **
    *
    ---Custom Field int values for project_details.financier
    primary_financier = [
    723
    ]





    ** Sunlight **
    *
    ---Custom Field int values for project_details.financier
    primary_financier = [
    19203
    ]





    ** Cash ** 
    *
    ---Custom Field int values for project_details.financier
    primary_financier = 721




    '''

    # Initialize variables
    try:
        _data = customer_info

        # Create and assign dictionary for attributes of different loan products
        _loan_product_dictionary = {}

        _loanpal_25yr_199 = {
            'loan_product_name': '_loanpal_25yr_199',
            'product_term': '25',
            'product_apr': '1.99',
            'financier': 'LoanPal',
            'dropdown_name': '25 year, 1.99%'
        }
        _loan_product_dictionary['_loanpal_25yr_199'] = _loanpal_25yr_199

        _loanpal_25yr_299 = {
            'loan_product_name': '_loanpal_25yr_299',
            'product_term': '25',
            'product_apr': '2.99',
            'financier': 'LoanPal',
            'dropdown_name': '25 year, 2.99%'
        }
        _loan_product_dictionary['_loanpal_25yr_299'] = _loanpal_25yr_299

        _loanpal_25yr_198 = {
            'loan_product_name': '_loanpal_25yr_198',
            'product_term': '25',
            'product_apr': '1.98',
            'financier': 'LoanPal',
            'dropdown_name': '25 year, 1.98%'
        }
        _loan_product_dictionary['_loanpal_25yr_198'] = _loanpal_25yr_198

        _loanpal_20yr_149 = {
            'loan_product_name': '_loanpal_20yr_149',
            'product_term': '20',
            'product_apr': '1.49',
            'financier': 'LoanPal',
            'dropdown_name': '20 year, 1.49%'
        }
        _loan_product_dictionary['_loanpal_20yr_149'] = _loanpal_20yr_149

        _loanpal_15yr_499 = {
            'loan_product_name': '_loanpal_15yr_499',
            'product_term': '15',
            'product_apr': '4.99',
            'financier': 'LoanPal',
            'dropdown_name': '15 year, 4.99%'
        }
        _loan_product_dictionary['_loanpal_15yr_499'] = _loanpal_15yr_499

        _loanpal_12yr_299 = {
            'loan_product_name': '_loanpal_12yr_299',
            'product_term': '12',
            'product_apr': '2.99',
            'financier': 'LoanPal',
            'dropdown_name': '12 year, 2.99%'
        }
        _loan_product_dictionary['_loanpal_12yr_299'] = _loanpal_12yr_299

        _loanpal_10yr_499 = {
            'loan_product_name': '_loanpal_10yr_499',
            'product_term': '10',
            'product_apr': '4.99',
            'financier': 'LoanPal',
            'dropdown_name': '10 year, 4.99%'
        }
        _loan_product_dictionary['_loanpal_10yr_499'] = _loanpal_10yr_499

        _loanpal_7yr_699 = {
            'loan_product_name': '_loanpal_7yr_699',
            'product_term': '7',
            'product_apr': '6.99',
            'financier': 'LoanPal',
            'dropdown_name': '7 year, 6.99%'
        }
        _loan_product_dictionary['_loanpal_7yr_699'] = _loanpal_7yr_699

        _sunlight_20yr_099 = {
            'loan_product_name': '_sunlight_20yr_099',
            'product_term': '20',
            'product_apr': '0.99',
            'financier': 'Sunlight'
        }
        _loan_product_dictionary['_sunlight_20yr_099'] = _sunlight_20yr_099

        _mosaic_10yr_399 = {
            'loan_product_name': '_mosaic_10yr_399',
            'product_term': '10',
            'product_apr': '3.99',
            'financier': 'Mosaic'
        }
        _loan_product_dictionary['_mosaic_10yr_399'] = _mosaic_10yr_399

        _mosaic_15yr_399 = {
            'loan_product_name': '_mosaic_15yr_399',
            'product_term': '15',
            'product_apr': '3.99',
            'financier': 'Mosaic'
        }
        _loan_product_dictionary['_mosaic_15yr_399'] = _mosaic_15yr_399

        _mosaic_20yr_499 = {
            'loan_product_name': '_mosaic_20yr_499',
            'product_term': '20',
            'product_apr': '4.99',
            'financier': 'Mosaic'
        }
        _loan_product_dictionary['_mosaic_20yr_499'] = _mosaic_20yr_499

        _mosaic_25yr_199 = {
            'loan_product_name': '_mosaic_25yr_199',
            'product_term': '25',
            'product_apr': '1.99',
            'financier': 'Mosaic'
        }
        _loan_product_dictionary['_mosaic_25yr_199'] = _mosaic_25yr_199

        _mosaic_25yr_299 = {
            'loan_product_name': '_mosaic_25yr_299',
            'product_term': '25',
            'product_apr': '2.99',
            'financier': 'Mosaic'
        }
        _loan_product_dictionary['_mosaic_25yr_299'] = _mosaic_25yr_299

        _cash_00_000 = {
            'loan_product_name': '_cash',
            'product_term': '0',
            'product_apr': '0',
            'financier': 'Customer'
        }
        _loan_product_dictionary['_cash'] = _cash_00_000
    except Exception as e:
        print(f'While attempting to intialize variables and dictionaries for structuring the Loan Product information,'
              f'the following error occurred: {e}')
        # TODO - LOG ERROR
        return -1
    # print(_data)

    # Determine and assign value for loan_product
    try:
        # Only want to run financier portal checks if all required information is present

        if _data['financier'] == 'Cash':
            loan_product = '_cash'

        elif _data['financier'] is not None and _data['loan_term'] is not None \
                and _data['interest_rate'] is not None and _data['loan_amount'] is not None:

            # print('Info there, now attempting to check loan')

            ###### LOANPAL ######
            if _data['financier'] == 'LoanPal' or _data['financier'] == 'GoodLeap':
                ### 25 YEAR LOANS ###
                if _data['loan_term'] == '25':
                    # _data['loan_term'] == '428' or

                    # 25 year, 2.99% BP or BP+ Loans
                    if _data['interest_rate'] == '0.03' or _data['interest_rate'] == '2.99':
                        loan_product = '_loanpal_25yr_299'
                        dropdown_name = '25 year, 2.99%'

                    # 25 year, 1.98% SmartStart loan
                    elif _data['interest_rate'] == '1.98':
                        loan_product = '_loanpal_25yr_198'
                        dropdown_name = '25 year, 1.98%'

                    # 25 year, 1.99% BP or BP+ Loans
                    else:
                        loan_product = '_loanpal_25yr_199'
                        dropdown_name = '25 year, 1.99%'

                #### 20 YEAR LOANS ###
                # 20 Year, 1.49% BP or BP+ Loans
                elif _data['loan_term'] == '20':
                    # _data['loan_term'] == '427':
                    loan_product = '_loanpal_20yr_149'
                    dropdown_name = '20 year, 1.49%'

                ### 15 YEAR LOANS ###
                # 15 Year, 4.99%  BP or BP+ Loans
                elif _data['loan_term'] == '15':
                    # _data.get('loan_term') == '426':
                    loan_product = '_loanpal_15yr_499'
                    dropdown_name = '15 year, 4.99%'

                ### 12 YEAR LOANS ###
                # 12 Year, 2.99%  BP or BP+ Loans
                elif _data['loan_term'] == '12':
                    loan_product = '_loanpal_12yr_299'
                    dropdown_name = '12 year, 2.99%'


                ### 10 YEAR LOANS ###
                # 10 Year, 4.99%  BP or BP+ Loans
                elif _data['loan_term'] == '10':
                    loan_product = '_loanpal_10yr_499'
                    dropdown_name = '10 year, 4.99%'

                ### 7 YEAR LOANS ###

                elif _data['loan_term'] == '7' or _data[
                    'loan_term'] == '07':  # 7 Year Loan Term has internal CFGA LOV ID of 19110,
                    loan_product = '_loanpal_7yr_699'  # APR of 6.99 as of 3/22/21
                    dropdown_name = '7 year, 6.99%'

            ###### MOSAIC ######
            elif _data['financier'] == 'Mosaic':
                return -5

            ###### SUNLIGHT ######
            elif _data['financier'] == 'Sunlight':
                return -6

            ###### CASH ######
            elif _data['financier'] == 'Cash':
                loan_product = '_cash'
        # elif _value_dictionary[_data['financier']] == 'Cash':
        #     loan_product = '_cash'
        else:
            return -2
    except Exception as e:
        print(f'While attempting to structure loan product information, the following error occurred: {e}')
        # TODO - LOG ERROR
        return -3

    # Return the structured dictionary of loan product information
    try:
        # print(_loan_product_dictionary[loan_product])
        information_to_return = _loan_product_dictionary[loan_product]
        # print(information_to_return)
        return information_to_return
    except Exception as e:
        print(f'An error occurred when attempting to return the structured loan product information: {e}')
        # TODO - LOG ERROR
        return -10


