# **********************************************************************************************************************
# *                                                                                                                    *
# *  PandaDoc Interaction Module                                                                                       *
# *  Created by Chris B for Blue Raven Solar, LLC                                                                      *
# *  V 1.00.00                                                                                                         *
# *  9/27/2021                                                                                                         *
#                                                                                                                      *
# **********************************************************************************************************************


# @@@@@
#   @@@@@@@@
#    @@@@@@@@@@@.
#     @@@@@@@@@@@@@@,
#      @@@@@@@@@@@@@@@@@,
#       /@@@@@@@@@@@@@@@@@@@.
#         @@@@@@@@@@@@@@@@@@@@@@
#           @@@@@@@@@@@@@@@@@@@@@@@@
#             @@@@@@@@@@@@@@@@@@@@@@@@@@
#                ,@@@@@@@@@@@@@@@@@@@@@@@@@@
#                      #@@@@@@@@@@@@@@@@@@@@@@@@
#                            #@@@@@@@@@@@@@@@@@@@@@@.
#                     @@.          .@@@@@@@@@@@@@@@@@@@@.
#                       @@@@@@@@@         @@@@@@@@@@@@@@@@@@
#                        .@@@@@@@@@@@@@@@       @@@@@@@@@@@@@@@@,                                                   &@@@
#                           @@@@@@@@@@@@@@@@@@@@%     @@@@@@@@@@@@@@                                      %@@@@@@@@@@@@
#                              @@@@@@@@@@@@@@@@@@@@@@@@@*  *@@@@@@@@@@@                        #@@@@@@@@@@@@@@@@@@@@@
#                                   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@              &@@@@@@@@@@@@@@@@@@@@@@@@@
#                                             #@@@@@@@@@@@@@@@@@@@@@@@@@@@         @@@@@@@@@@@@@@@@@@@@#(
#                                     &@@@@@@@...       ,@@@@@@@@@@@@@@@@@      @@@@@@@@@@@@@@@%..@@@@@@@@@
#                                       @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ (@@@@@@@@@@@@@@@@@@@@@@@@@@%
#                                           @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@.
#                                                     ,@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#                                                @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@***@@@@**
#                                                   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
#                                                                 ##@@@@@@@@@@@@@@@@@@@@#
#                                                                #@@@@@@@@@@@
#                                                              @@@@@@@@@@@@@
#                                                              @@@@@@@@@@@@
#
#
#            ____  _                ____                            ____        _
#           | __ )| |_   _  ___     |  _ \ __ ___   _____ _ __      / ___|  ___ | | __ _ _ __
#           |  _ \| | | | |/ _ \    | |_) / _` \ \ / / _ \ '_ \     \___ \ / _ \| |/ _` | '__|
#           | |_) | | |_| |  __/    |  _ < (_| |\ V /  __/ | | |     ___) | (_) | | (_| | |
#           |____/|_|\__,_|\___|    |_| \_\__,_| \_/ \___|_| |_|    |____/ \___/|_|\__,_|_|
#           ---------------------------------------------------------------------------------
#

# Import local modules
#try:
#    import mail_helper
#except ImportError:
#    from .. import email

try:
    import sys
    import os
    # import BRSoptions as brs_o
    # from BRSoptions import alb_configs as alb_con
    # from BRSoptions import external_configs as ex_con
    # import BRSDBQ
    OG_REFS = True
except ImportError:
    print('Error importing local modules')
    from ...util import config as configs
    from ...util.data import Auth as auth
    from ...util.data import BRSValues as brs_vals
    from ...util.data import ThirdPartyValues as ext_vals
    from ...ALB import db_conn as dbq

    OG_REFS = False
except Exception as e:
    print(f'The following error occurred while importing local modules: {e}')
    sys.exit(-2)

# Import non-local modules
try:
    from selenium import webdriver
    from selenium.webdriver.common.keys import Keys
    from selenium.webdriver.support.select import Select
    from selenium.webdriver.support.ui import WebDriverWait

    import datetime
    from datetime import date
    import time
    import requests
    import json
    import urllib
    import hashlib
    try:
        from requests_oauthlib import OAuth2Session
    except ImportError:
        os.system('pip install requests_oauthlib')
        from requests_oauthlib import OAuth2Session
    except Exception as e:
        print(f'The following unexpected error occurred when attempting to import to OAuth 2 module: {e}')


except ImportError:
    print('Error importing non-local modules')
    sys.exit(-3)

except Exception as e:
    print(f'The following error occurred while importing non-local modules: {e}')
    sys.exit(-4)


# ASSIGN VALUES TO VARIABLES THAT ARE GLOBAL TO THIS MODULE
try:
    if OG_REFS:
        windowless_chrome_mode = alb_con.web_driver_hidden
        non_expiring_token = alb_con.albatross_non_expiring_token
        alt_non_expiring_alb_toke = alb_con.alternate_albatross_non_expiring_token
        alb_bearer_token = alb_con.albatross_bearer_token
    else:
        current_configs = configs.DefaultConfigs()
        windowless_chrome_mode = current_configs.web_driver_hidden
        non_expiring_token = auth.albatross_non_expiring_token
        alt_non_expiring_alb_toke = auth.alternate_albatross_non_expiring_token
        alb_bearer_token = auth.albatross_bearer_token

except ValueError:
    print('Value Error occurred when attempting to set PandaDoc module global variables')
    sys.exit(-5)

except TypeError:
    print('Type Error occurred when attempting to set PandaDoc module global variables')
    sys.exit(-6)

except NameError as eN:
    print('Name Error occurred when attempting to set PandaDoc module global variables')
    sys.exit(eN)
except Exception as e:
    print(f'The following unexpected error/exception occurred: {e}')
    sys.exit(-7)

class PandaDocSession:

    def __init__(self, brs_user_profile=None, **kwargs):

        if brs_user_profile is None:
            raise TypeError('No user profile provided - PandaDocSession cannot initialize.')
        else:
            self.user_profile = brs_user_profile

        self.login_url = 'https://app.pandadoc.com/login/'
        self.api_endpoint = r"https://api.pandadoc.com"

        if brs_user_profile == 'Sales HR':
            self.login_name = 'saleshr@blueravensolar.com'
            self.login_password = 'PolarBear2015!'

            try:
                self.prod_api_key = ex_con.pandadoc_prod_api_key
                self.sandbox_api_key = ex_con.pandadoc_sandbox_api_key
                self.application_client_id = ex_con.pandadoc_MyTestApp_client_id
                self.application_client_secret = ex_con.pandadoc_MyTestApp_client_secret
            except:
                self.prod_api_key = ext_vals.PandaDoc.pandadoc_prod_api_key
                self.sandbox_api_key = ext_vals.PandaDoc.pandadoc_sandbox_api_key
                self.application_client_id = ext_vals.PandaDoc.pandadoc_MyTestApp_client_id
                self.application_client_secret = ext_vals.PandaDoc.pandadoc_MyTestApp_client_secret

        elif brs_user_profile == 'Permitting' or brs_user_profile == 'Corporate':
            self.login_name = 'permitting.department@blueravensolar.com'
            self.login_password = 'PolarBear2015!'

        elif brs_user_profile == 'HR':
            self.login_name = 'hr@blueravensolar.com'
            self.login_password = 'BrsHR.2018'

        else:
            raise ValueError('The entered user profile is not a valid profile.')

        try:
            self.template_list = self.get_list_of_templates()
        except Exception as e:
            print(f'While attempting to fetch the list of templates, the following error occurred: {e}')

    def get_list_of_templates(self,
                              panda_client_id=None,
                              panda_client_secret=None,
                              panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"
                              ):
        import BRS_TK.util.data.Auth as ext_vals

        if panda_client_id is None:
            _client_id =  ext_vals.pandadoc_MyTestApp_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = ext_vals.pandadoc_MyTestApp_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        try:
            _api_key = ext_vals.pandadoc_prod_api_key
        except Exception as e:
            sys.exit(e)

        _payload = r'client_id=' + _client_id + \
                   r'client_secret=' + _client_secret + \
                   r'scope=read%2Bwrite&' + \
                   r'redirect_uri=' + _redirect_uri + \
                   r'api_key=' + _api_key

        _headers = {}
        _headers.update({'Authorization': f'API-Key c0c762b50131b4128d16760a8cdec6ec3a5ea361'})
        _headers.update({'Content-Type': 'application/json'})

        _response = requests.request(
            "GET",
            url=r"http://api.pandadoc.com/public/v1/templates",
            headers=_headers,
            data=_payload
        )

        _template_list_dict = json.loads(_response.text)
        _template_list = _template_list_dict['results']

        return _template_list

    def find_template_by_name(self,
                              template_name=None,
                              panda_client_id=None,
                              panda_client_secret=None,
                              panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"
                              ):

        if template_name is None:
            raise TypeError
        else:
            _template_name = template_name

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        template_to_return = None
        _temp_template_list = self.template_list

        for template in _temp_template_list:
            if template['name'] == _template_name:
                template_to_return = template
                break

        return template_to_return

    def find_template_by_id(self,
                            template_uuid=None,
                            panda_client_id=None,
                            panda_client_secret=None,
                            panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"
                            ):

        if template_uuid is None:
            raise TypeError
        else:
            _template_id = template_uuid

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        template_to_return = None
        _temp_template_list = self.template_list

        for template in _temp_template_list:
            if template['id'] == _template_id:
                template_to_return = template
                break

        return template_to_return

    def get_template_details(self,
                             target_template_id=None,
                             panda_client_id=None,
                             panda_client_secret=None,
                             panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"
                             ):

        if target_template_id is None:
            raise TypeError
        else:
            _template_id = target_template_id

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        _payload = r'client_id=' + _client_id + \
                   r'client_secret=' + _client_secret + \
                   r'scope=read%2Bwrite&' + \
                   r'redirect_uri=' + _redirect_uri + \
                   r'api_key=' + _api_key

        _headers = {}

        _headers.update({
            'Authorization': f'API-Key {_api_key}'
        })

        _headers.update({
            'Content-Type': 'application/json'
        })

        _response = requests.request(
            "GET",
            url=_redirect_uri + _template_id + r"/details",
            headers=_headers,
            data=_payload
        )

        return _response

    def create_new_document_from_template(self,
                                          document_name=None,
                                          template_uuid=None,
                                          folder_id=None,
                                          recipients=None,
                                          field_values_dictionary=None,
                                          panda_client_id=None,
                                          panda_client_secret=None,
                                          panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"):

        if None in (document_name, template_uuid, folder_id, recipients, field_values_dictionary):
            raise TypeError('You must pass a value for all required parameters: '
                            'document_name, template_id, folder_id, recipients, and field_values_dictionary')
        else:
            _document_name = document_name
            _template_id = template_uuid
            _folder_id = folder_id
            _recipients = recipients
            _values = field_values_dictionary

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        _actual_api_uri = r"https://api.pandadoc.com/public/v1/documents"

        _payload = {
            'name': _document_name,
            'template_uuid': _template_id,
            'folder_uuid': _folder_id,
            'recipients': _recipients,
            'fields': _values,
            'client_id': _client_id,
            'client_secret': _client_secret,
            'redirect_uri': _redirect_uri,
            'api_key': _api_key
        }

        _headers = {}
        _headers.update({
            'Authorization': f'API-Key {_api_key}'
        })

        _headers.update({
            'Content-Type': 'application/json'
        })

        _response = requests.request(
            "POST",
            url=_actual_api_uri,
            headers=_headers,
            json=_payload
        )

        _document_response_info = json.loads(_response.text)
        return _document_response_info

    def get_document_status(self,
                            document_id=None,
                            panda_client_id=None,
                            panda_client_secret=None,
                            panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"):

        if document_id is None:
            raise TypeError
        else:
            _document_id = document_id

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        _actual_api_uri = r"https://api.pandadoc.com/public/v1/documents/" + _document_id

        _payload = {'client_id': _client_id,
                    'client_secret': _client_secret,
                    'redirect_uri': _redirect_uri,
                    'api_key': _api_key,
                    'id': _document_id}

        _headers = {}
        _headers.update({
            'Authorization': f'API-Key {_api_key}'
        })

        _headers.update({
            'Content-Type': 'application/json'
        })

        _response = requests.request(
            "GET",
            url=_actual_api_uri,
            headers=_headers,
            json=_payload
        )

        _document_status_response = json.loads(_response.text)

        return _document_status_response

    def get_item_content_details(self,
                                 item_id=None,
                                 panda_client_id=None,
                                 panda_client_secret=None,
                                 panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"):

        if item_id is None:
            raise TypeError
        else:
            _item_id = item_id

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        _actual_api_uri = r"https://api.pandadoc.com/public/v1/content-library-items/" + _item_id + r"/details"

        _payload = {
            'client_id': _client_id,
            'client_secret': _client_secret,
            'redirect_uri': _redirect_uri,
            'api_key': _api_key,
            'uuid': _item_id
        }

        _headers = {}
        _headers.update({
            'Authorization': f'API-Key {_api_key}'
        })
        _headers.update({
            'Content-Type': 'application/json'
        })

        _response = requests.request(
            "GET",
            url=_actual_api_uri,
            headers=_headers,
            json=_payload
        )

        _content_details_response = json.loads(_response.text)

        return _content_details_response

    def send_document(self,
                      document_id=None,
                      form_data=None,
                      panda_client_id=None,
                      panda_client_secret=None,
                      panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"):

        if document_id is None:
            raise TypeError
        else:
            _document_id = document_id

        _api_key = self.prod_api_key

        if panda_client_id is None:
            _client_id = self.application_client_id
        else:
            _client_id = panda_client_id

        if panda_client_secret is None:
            _client_secret = self.application_client_secret
        else:
            _client_secret = panda_client_secret

        if panda_redirect_uri is None:
            _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
        else:
            _redirect_uri = panda_redirect_uri

        _email_subject_line = r'Blue Raven Solar Setter/Closer Agreement'
        _email_message = r'Welcome to Blue Raven Solar! We are excited to have you as part of the BRS team. ' \
                         r'Check out our sales rep brochure for information on Blue Raven Solar, here! ' \
                         r'http://blueravensolar.com/brochure/ Attached is your Rep Agreement through PandaDoc. ' \
                         r'You will also be directed to attach a copy of a voided check or a screen shot of your ' \
                         r'routing & account numbers. NOTE: you will not be able to complete this packet without ' \
                         r'the attachment, PLEASE DO NOT SELECT "Send as Fax" as we do not have a fax machine and ' \
                         r'we will not be able to receive your completed agreement if you select this option. ' \
                         r'Thank you, Blue Raven Solar saleshr@blueravensolar.com 385-498-6176'

        _actual_api_uri = r"https://api.pandadoc.com/public/v1/documents/" + _document_id + r"/send"

        _payload = {'client_id': _client_id,
                    'client_secret': _client_secret,
                    'redirect_uri': _redirect_uri,
                    'api_key': _api_key,
                    'uuid': _document_id,
                    'subject': _email_subject_line,
                    'message': _email_message}

        _headers = {}
        _headers.update({
            'Authorization': f'API-Key {_api_key}'
        })
        _headers.update({
            'Content-Type': 'application/json'
        })

        _response = requests.request(
            "POST",
            url=_actual_api_uri,
            headers=_headers,
            json=_payload
        )

        _document_sent_response = json.loads(_response.text)

        return _document_sent_response



def generate_oauth_redirect_uri(panda_client_id=None,
                                panda_client_secret=None,
                                panda_redirect_uri=None
                                ):
    if panda_client_id is None:
        _client_id = auth.pandadoc_MyTestApp_client_id
    else:
        _client_id = panda_client_id

    if panda_client_secret is None:
        _client_secret = auth.pandadoc_MyTestApp_client_secret
    else:
        _client_secret = panda_client_secret

    if panda_redirect_uri is None:
        _redirect_uri = "https://api.pandadoc.com/public/v1/templates?" \
                        "q=onboarding&" \
                        "page=1&" \
                        "deleted=False&" \
                        "folder_uuid=BVvPPtGKxWd7u9F8wMrSgR"
    else:
        _redirect_uri = panda_redirect_uri

    init_url = f'https://app.pandadoc.com/oauth2/authorize?' \
               f'client_id={_client_id}&' \
               f'redirect_uri={_redirect_uri}&' \
               f'scope=read+write&' \
               f'response_type=code'

    return init_url


def generate_state_token():
    _state = hashlib.sha256(os.urandom(1024)).hexdigest()
    _cur_session = requests.Session()
    _cur_session['state'] = _state




def get_access_token(panda_client_id=None,
                     panda_client_secret=None,
                     panda_redirect_uri=r"https://api.pandadoc.com/public/v1/templates"
                     ):

    if panda_client_id is None:
        _client_id = auth.pandadoc_MyTestApp_client_id
    else:
        _client_id = panda_client_id

    if panda_client_secret is None:
        _client_secret = auth.pandadoc_MyTestApp_client_secret
    else:
        _client_secret = panda_client_secret

    if panda_redirect_uri is None:
        _redirect_uri = r"https://api.pandadoc.com/public/v1/templates"
    else:
        # _oauth_url = panda_oauth_url
        _redirect_uri = panda_redirect_uri

    # oauth_session = OAuth2Session(_client_id,
    #                              redirect_uri=_oauth_url,
    #                              scope=r"read+write"
    #                              )
    #
    # authorization_url, state = oauth_session.authorization_url(r"https://app.pandadoc.com/oauth2/authorize")
    #print(authorization_url)
    # _data = {'grant_type': 'authorization_code',
    #          'code': '40390dee725b68d39eef8b1e1c758009664f8896',
    #          'redirect_uri': r"https://api.pandadoc.com/public/v1/templates"
    #          }
    # _access_token_url = r"https://api.pandadoc.com/oauth2/access_token"
    # _payload =r'grant_type=authorization_code&' \
    #           r'client_id=b9221d4ae482c7a09ce0&' \
    #           r'client_secret=384daa8e6effbd5d57dae4432f0a075d3ddea9ca&' \
    #           r'code=2a7b379a3c5e7a3ab5d1d166f40c69632ca38093&' \
    #           r'scope=read%2Bwrite&' \
    #           r'redirect_uri=https%3A%2F%2Fapi.pandadoc.com/public/v1/templates'
    _payload2 = r'client_id=b9221d4ae482c7a09ce0&' \
                r'client_secret=384daa8e6effbd5d57dae4432f0a075d3ddea9ca&' \
                r'scope=read%2Bwrite&' \
                r'redirect_uri=https%3A%2F%2Fapi.pandadoc.com/public/v1/templates' \
                r'api_key=c0c762b50131b4128d16760a8cdec6ec3a5ea361'

    headers = {}
    headers.update({'Authorization': f'API-Key c0c762b50131b4128d16760a8cdec6ec3a5ea361'})
    headers.update({'Content-Type': 'application/json'})
    # _response1 = requests.request("POST", url=_access_token_url, headers=headers, data=_payload)
    _response2 = requests.request("GET",
                                  url=r"http://api.pandadoc.com/public/v1/templates",
                                  headers=headers,
                                  data=_payload2)
    print(_response2.text)
    print(_response2)
    #print(_response1.text)
    #_access_token_response = requests.post()
    return _response2


def get_templates(search_query=None,
                  search_tag=None,
                  return_count=50,
                  page_to_return=1,
                  template_deleted=False,
                  template_uuid=None,
                  folder_uuid=None,
                  open_session=None):

    # NO PARAMETERS CAN BE BLANK
    # Record the parameters of the API call

    # PARAMETERS
    _search_query = search_query  # STRING - Optional search query to filter by template name
    _search_tag = search_tag  # STRING - Optional search tag to filter by template tag
    _return_count = return_count  # INT32 - How many templates to return. Default is 50, max is 100
    _page_to_return = page_to_return  # INT32 - Optionally specify which page of the dataset to return
    _template_deleted = template_deleted  # BOOLEAN - Specify whether to return templates that have or haven't been deleted
    _template_id = template_uuid  # STRING - Optionally specify tempalte ID
    _folder_uuid = folder_uuid  # STRING - The UUID of the folder where tempaltes are stored

    # _search_query_url_chunk = r""
    #     , _search_tag_url_chunk, _return_count_url_chunk, _page_to_return_url_chunk = r""
    # _template_deleted_url_chunk, _template_id_url_chunk, _folder_uuid_url_chunk = r""

    # HEADERS
    if OG_REFS:
        pandadoc_auth_token = ex_con.pandadoc_prod_api_key
    else:
        pandadoc_auth_token = auth.pandadoc_prod_api_key

    # Initialize session
    if open_session is None:
        pandadoc_session = requests.Session()
        pandadoc_session.headers.update({'Authorization': f'API-Key {pandadoc_auth_token}'})
    else:
        pandadoc_session = open_session
        pandadoc_session.headers.update({'Authorization': f'API-Key {pandadoc_auth_token}'})


    # PARAMETERS

    param_dictionary = {}
    api_url_append_str = r""
    if _search_query is None:
        _search_query_url_chunk = r""
    else:
        _search_query_url_chunk = r"q=" + _search_query + r""
        api_url_append_str += _search_query_url_chunk
        print(_search_query_url_chunk)
        param_dictionary['q'] = _search_query

    if _search_tag is None:
        _search_tag_url_chunk = r""
    else:
        _search_tag_url_chunk = r"&tag=" + _search_tag + r""
        api_url_append_str += _search_query_url_chunk
        param_dictionary['tag'] = _search_tag

    if _return_count is None or _return_count == 50:
        _return_count_url_chunk = r""
    else:
        _return_count_url_chunk = r"&count=" + str(_return_count) + r""
        api_url_append_str += _return_count_url_chunk
        param_dictionary['count'] = str(_return_count)

    if _page_to_return is None:
        _page_to_return_url_chunk = r""
    else:
        _page_to_return_url_chunk = r"&page=" + str(_page_to_return) + r""
        api_url_append_str += _page_to_return_url_chunk
        param_dictionary['page'] = str(_page_to_return)

    if _template_deleted is None:
        _template_deleted_url_chunk = r""
    else:
        _template_deleted_url_chunk = r"&deleted=" + str(_template_deleted) + r""
        api_url_append_str += _template_deleted_url_chunk
        param_dictionary['deleted'] = str(_template_deleted)

    if _template_id is None:
        _template_id_url_chunk = r""
    else:
        _template_id_url_chunk = r"&id=" + _template_id + r""
        api_url_append_str += _template_id_url_chunk
        param_dictionary['id'] = _template_id

    if _folder_uuid is None:
        _folder_uuid_url_chunk = r""
    else:
        _folder_uuid_url_chunk = r"&folder_uuid=" + _folder_uuid + r""
        api_url_append_str += _folder_uuid_url_chunk
        param_dictionary['folder_uuid'] = _folder_uuid

    # Format for URL
    api_url_append_str = api_url_append_str.replace(' ', r"%20")

    get_template_api_url = r"https://api.pandadoc.com/public/v1/templates?" + api_url_append_str
                           # + _search_query_url_chunk \
                           # + _search_tag_url_chunk \
                           # + _return_count_url_chunk \
                           # + _page_to_return_url_chunk \
                           # + _template_deleted_url_chunk \
                           # + _template_id_url_chunk \
                           # + _folder_uuid_url_chunk
    request_params = [param_dictionary]
    print(get_template_api_url)
    print(request_params)
    print(pandadoc_session.headers)
    try:
        session_response = pandadoc_session.get(
            f'{get_template_api_url}',
            headers={
                'Content-Type': 'application/json',
                'Connection': 'close',
                'Authorization': f'API-Key {pandadoc_auth_token}'
            },
            json=request_params
        )

        print(f'Attempted to search for templates - session response is {session_response}')
        return session_response

    except Exception as e:
        print(f'The exception {e} occurred while attempting to search for a template via PandaDoc API')
        return -1



if __name__ == '__main__':

    test_session = PandaDocSession('Sales HR')
    test_recipient_list = [
        {
            'email': 'christopher.bahr@blueravensolar.com',
            'first_name': 'Chris',
            'last_name': 'Bahr',
            'role': 'BRS Rep'
        },
        {
            'email': 'saleshr@blueravensolar.com',
            'first_name': 'Sales',
            'last_name': 'HR',
            'role': 'Sales HR'
        }
    ]
    current_date = datetime.date.today().strftime('%Y-%m-%d') + r'T00:00:00.000Z'

    test_values_dict = {
        'Employee.FirstName': {
            'value': 'Christopher'
        },
        'Employee.LastName': {
            'value': 'Bahr'
        },
        'Date': {
            'value': current_date
        }
    }

    email_template_file_path = brs_o.local_path + '/resources/non_co_closer_tempalte.html'
    try:
        with open(email_template_file_path) as template_file:
            template = template_file.read().format(name=test_values_dict['Employee.FirstName'],
                                                   email=test_recipient_list['email'])
    except Exception as e:
        print(f'{e} -- Excpetion occurred when trying to parse the non_co_closer_template html template')
        sys.exit(-1)

    # test_session = mail_helper.saleshr_device_flow_session()
    # sample_email = mail_helper.send_sales_mail(session = )




    sys.exit()
    template_dict = test_session.find_template_by_name('Blue Raven Solar Closer Agreement')
    template_id = template_dict['id']
    sample_new_doc = test_session.create_new_document_from_template(
        document_name=r'Testing Onboarding Docs',
        template_uuid=template_id,
        folder_id=r'k3FfRnFsaBts4PgJZ3VRAU',
        recipients=test_recipient_list,
        field_values_dictionary=test_values_dict
    )
    print(sample_new_doc)
    new_doc_id = sample_new_doc['id']

    ready_to_send = False
    timer = 0

    while ready_to_send is False and timer < 10:
        current_status = test_session.get_document_status(document_id=new_doc_id)

        if current_status['status'] == 'document.draft':
            ready_to_send = True
        else:
            timer += 1
            time.sleep(1)

    documents_sent = test_session.send_document(document_id=new_doc_id)
    print(documents_sent)


