import sys
import os

try:
    from thefuzz import fuzz
    from thefuzz import process
except ImportError:
    os.system('pip install thefuzz')
    from thefuzz import fuzzywuzzy
    from thefuzz import process
except NameError:
    os.system('pip install thefuzz')
    from thefuzz import fuzzywuzzy
    from thefuzz import process
except Exception as e:
    sys.exit(f'Error on importing fuzzy string stuff - {e}')


def find_best_match(stringToMatch=None,
                    listToCheck=[],
                    **kwargs):
    """
    Calculates the fuzzy matching scores for all of the strings in listToCheck compared to stringToMatch, and returns
    the string with the highest score.
    Optional - You can pass strings in the string_ignore keyword param, as a list.
    :param stringToMatch:
    :param listToCheck:
    :param kwargs:
    :return:
    """

    if None in (stringToMatch, listToCheck):
        raise ValueError('No values passed for the stringToMatch or the listToCheck')

    try:
        if kwargs['matchThreshold'] is not None:
            _match_threshold = kwargs['matchThreshold']
        else:
            _match_threshold = 60
    except KeyError as eK:
        # print(eK)
        _match_threshold = 60
    except Exception as e:
        print(f'Unexpected exception {e} while setting match threshold for find best match function')
        _match_threshold = 60

    ignore_words = False
    _words_to_ignore = []

    if kwargs.get('string_ignore', None) is not None:
        ignore_words = True
        _words_to_ignore = kwargs['string_ignore']


    _target_string = stringToMatch
    _list_to_check = listToCheck
    _flagged_strings = []
    _master_list = []


    # Create list of strings to not check since they match an ignore_word argument
    if ignore_words == True:
        _ignore_list_fuzzy = [abc[0] for abc in _words_to_ignore if abc[1] is False]
        _ignore_list_exact = [abc[0] for abc in _words_to_ignore if abc[1] is not False]

        for chk_str in _list_to_check:

            _compare_list = []
            _compare_list = process.extract(chk_str, _ignore_list_fuzzy)

            if max([abc[1] for abc in _compare_list]) > _match_threshold:
                _flagged_strings.append({'flagged_string': chk_str,
                                         'match_score': max([abc[1] for abc in _compare_list]),
                                         'matched_word': _compare_list[[abc[1] for abc in _compare_list].index(max([abc[1] for abc in _compare_list]))][0]
                                         })

            if len(_ignore_list_exact) > 0:

                import re
                for _exact_match_word in _ignore_list_exact:
                    _regex_str = r"(" + _exact_match_word + r")"
                    _re = re.compile(_regex_str)

                    if re.search(_regex_str, r"" + chk_str + r"") is not None:
                        _flagged_strings.append({'flagged_string': chk_str,
                                                 'match_score': 999,
                                                 'matched_word': _exact_match_word
                                                 })

    # List of strings to check for a match, with the ones that matched ignore_words removed from the list
    # print(f'Flagged strings: {_flagged_strings}')
    _extracted_list_to_check = [
        abc for abc in _list_to_check if abc not in [
            abc['flagged_string'] for abc in _flagged_strings
        ]
    ]
    print(f'flagged strings: {_flagged_strings}')
    print(f'target_string {_target_string}')
    print(f'extracted list to check {_extracted_list_to_check}')

    if len(_extracted_list_to_check) < 2:
        return [_extracted_list_to_check, 100]

    _match_check = process.extract(_target_string, _extracted_list_to_check)
    print(_match_check)
    try:
        _best_matched_string = _match_check[[abc[1] for abc in _match_check].index(max(*[abc[1] for abc in _match_check]))]
        print(_best_matched_string)
    except ValueError as eV:
        print(eV)
        _match_check = False
    except Exception as e:
        print(e)

    # _best_match = _best_matched_string[0]
    _best_match = _best_matched_string
    return _best_match

if __name__ == '__main__':
    test_string = 'FD Final Design 4532 John Mcphersson'
    test_compare_strings = [
        'FD Design Tool 9.01 4532 John Mcphersson',
        'FD Shade Report 4532 John Mcphersson',
        'FD-AL-dxf.1633964650993',
        'FD PV Watts 4532 John Mcphersson',
        'FD 4532 John Mcphersson',
        'other test string(s)',
        'FD SS 4532 John Mcphersson'
    ]
    #print(find_best_match(test_string, test_compare_strings))

    words_to_ignore = [
        ('shade', False),
        ('tool', False),
        ('dxf', False),
        ('SS', True),
        ('pvwatts', False)
    ]
    print('\ntest_string ----')
    print(test_string)
    print('\ntest_compare_strings ----')
    print(test_compare_strings)
    print('\nwords_to_ignore')
    print(words_to_ignore)
    print('\n')
    match_attempt_one = find_best_match(test_string, test_compare_strings, string_ignore=words_to_ignore)

    if match_attempt_one is None:
        match_attempt_two = find_best_match(test_string,
                                            test_compare_strings,
                                            string_ignore=words_to_ignore,
                                            matchThreshold=60)
    print(match_attempt_one)
    sys.exit()