try:
    from .data import BRSValues as vals
except ImportError as eI:
    print('Error importing the data for the config file')
    import sys
    sys.exit(eI)
except Exception as e:
    import sys
    sys.exit(e)

try:
    localpath = vals.local_path
    resourcepath = vals.resource_path
    # print(f'config.py module has loaded the localpath as {localpath}')
except Exception as e:
    import sys
    sys.exit(e)

#class CurrentConfigs:
    
class ConfigSetup:

    def __init__(self, config_type=None, **kwargs):

        if config_type is not None:

            self.config_type = config_type

            if self.config_type == 'Default':
                self.load_default_configs()

            elif self.config_type == 'finch':
                self.load_finch_configs(**kwargs)

            else:
                self.load_default_configs()



        else:
            self.config_type = 'Default'

            if kwargs is None:
                self.load_default_configs()

            else:

                try:
                    self.web_driver_hidden = kwargs['web_driver_hidden']
                except KeyError:
                    self.web_driver_hidden = True
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.default_download_location = kwargs['default_download_location']
                except KeyError:
                    self.default_download_location = localpath + '/temp'
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.chrome_driver_path = kwargs['chrome_driver_path']
                except KeyError:
                    self.chrome_driver_path = resourcepath + "/chromedriver.exe"
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.kill_chrome_path = kwargs['kill_chrome_path']
                except KeyError:
                    self.chrome_driver_path = resourcepath + "/kill_chromedriver.bat"
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.testing_mode_enabled = kwargs['testing_mode_enabled']
                except KeyError:
                    self.testing_mode_enabled = False
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.batch_size = kwargs['batch_size_limit']
                except KeyError:
                    self.batch_size_limit = 50
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.console_line_length = kwargs['console_line_length']
                except KeyError:
                    self.console_line_length = 86
                except Exception as e:
                    print(f'Exception {e} occurred')

                try:
                    self.show_console = kwargs['show_console']
                except KeyError:
                    self.show_console = True
                except Exception as e:
                    print(f'Exception {e} occurred')


    def load_default_configs(self):
        self.web_driver_hidden = True
        self.default_download_location = localpath + '/temp'
        self.chrome_driver_path = resourcepath + "/chromedriver.exe"
        self.kill_chrome_path = resourcepath + "/kill_chromedriver.bat"
        self.testing_mode_enabled = False
        self.batch_size_limit = 50
        self.console_line_length = 86
        self.show_console = True
        self.note_title_default = 'FINAL DESIGN AUTO-SEND MASTER NOTE'

    def load_finch_configs(self, **kwargs):
        self.web_driver_hidden = True

        try:
            self.default_download_location = kwargs['default_download_location']
        except KeyError:
            self.default_download_location = localpath + '/temp'
        except Exception as e:
            print(f'Exception {e} occurred')


        self.chrome_driver_path = resourcepath + "/chromedriver.exe"
        self.kill_chrome_path = resourcepath + "/kill_chromedriver.bat"
        self.testing_mode_enabled = False
        self.testing_mode = self.testing_mode_enabled
        self.batch_size_limit = 50
        self.console_line_length = 86
        self.show_console = True
        self.note_title_default = 'FINAL DESIGN AUTO-SEND MASTER NOTE'
        self.current_finch_last_updated = '10-25-2021'
        self.current_finch_version = '2.00 beta'
        self.ready_to_send_fd_query = resourcepath + "/fd_auto_send_query.txt"
        self.bcc_sierra = False
        self.bcc_chris = True
        self.fd_email_template_file_path = resourcepath + "\\sendfdemail.html"
        self.fd_email_template_filepath = self.fd_email_template_file_path



        
class DefaultConfigs:

    def __init__(self):
        self.web_driver_hidden = True
        self.default_download_location = localpath + '/temp'
        self.chrome_driver_path = localpath + "/resources/chromedriver.exe"
        self.kill_chrome_path = localpath + "/resources/kill_chromedriver.bat"
        self.testing_mode_enabled = False
        self.batch_size_limit = 50
        self.console_line_length = 86
        self.show_console = True

class SalesOnboardingConfigs:

    def return_new_request_single_user_query_string(self,user_id=None):
        if user_id is None:
            raise TypeError('No user ID provided')
        else:
            _user_id_str = str(user_id)
            new_request_single_user_query = """WITH NEW_SALES_USERS AS (
                    with recruited_by as (
                        select 	
                            ucfv.int_value "recruiter_user_id",
                            u.first_name || ' ' || u.last_name "recruiter_name",
                            ucfv.user_id "user_id"
                        from
                            flow.user_custom_field_value ucfv
                        left join flow.user_position up on
                            up.id = ucfv.int_value
                        left join flow.position pos on
                            pos.id = up.position_id
                        left join flow.custom_field_group_assignment cfga on
                            cfga.id = ucfv.custom_field_group_assignment_id
                        left join flow.custom_field cf on
                            cf.id = cfga.custom_field_id
                        left join flow.user u on u.id = ucfv.int_value
                        where 
                            cfga.id = 330
                            and 
                            cf.id = 455
                    )
                    select 
                        DISTINCT u.id,
                        u.first_name,
                        u.last_name,
                        ust.user_status_type,
                        pos.position,
                        up.start_date,
                        up.end_date,
                        (uphv.hierarchy::json -> 4)::json ->> 'orgName' "area",
                        (uphv.hierarchy::json -> 4)::json ->> 'orgId' "area_id",
                        (uphv.hierarchy::json -> 4)::json ->> 'level' "area_level",
                        (uphv.hierarchy::json -> 5)::json ->> 'orgName' "region",
                        (uphv.hierarchy::json -> 5)::json ->> 'orgId' "region_id",
                        (uphv.hierarchy::json -> 5)::json ->> 'level' "region_level",
                        (uphv.hierarchy::json -> 6)::json ->> 'orgName' "district",
                        (uphv.hierarchy::json -> 6)::json ->> 'orgId' "district_id",
                        (uphv.hierarchy::json -> 6)::json ->> 'level' "district_level",
                        (uphv.hierarchy::json -> 7)::json ->> 'orgName' "office",
                        (uphv.hierarchy::json -> 7)::json ->> 'orgId' "office_id",
                        (uphv.hierarchy::json -> 7)::json ->> 'level' "office_level",
                        rb.recruiter_user_id "recruiter_user_id",
                        rb.recruiter_name "recruiter_name",
                        u.email "user_email",
                        u_rec.email "recruiter_email"

                    from flow.user u

                    inner join flow.user_position up on 
                        (up.user_id = u.id
                        and
                        up.primary_flag is True)

                    inner join flow.position pos on
                        pos.id = up.position_id

                    inner join flow.company com on
                        com.id = pos.company_id

                    inner join flow.org on
                        org.id = up.org_id

                    inner join flow.org_type ot on
                        ot.id = org.org_type_id

                    inner join flow.user_positions_vw upv
                        on upv.user_id = u.id

                    inner join flow.user_status_type ust
                        on ust.id = upv.user_status_type_id

                    inner join flow.user_position_hierarchy_vw uphv on
                        uphv.user_id = u.id

                    left join recruited_by rb on
                        rb.user_id = u.id

                    left join flow.user u_rec on
                        u_rec.id = rb.recruiter_user_id

                    where 
                        (""" + _user_id_str + """ = u.id)
                        or
                        (ust.user_status_type = 'New Request'
                        and
                        (
                            pos.position = 'Closer' 
                            or 
                            pos.position = 'Setter'
                        )
                        and
                        up.start_date is not null
                        and
                        up.end_date is null
                        and
                        up.archived is False
                        and
                        up.primary_flag is True)
                )
                Select 
                    row_to_json(NEW_SALES_USERS.*) "json_info_new_sales_user"

                from 
                    NEW_SALES_USERS"""

        return new_request_single_user_query

    def return_new_request_users_query(self):

        all_users_info_query_string = """
                    WITH NEW_SALES_USERS AS (
                        with recruited_by as (
                                select 	
                                    ucfv.int_value "recruiter_user_id",
                                    u.first_name || ' ' || u.last_name "recruiter_name",
                                    ucfv.user_id "user_id"
                                from
                                    flow.user_custom_field_value ucfv
                                left join flow.user_position up on
                                    up.id = ucfv.int_value
                                left join flow.position pos on
                                    pos.id = up.position_id
                                left join flow.custom_field_group_assignment cfga on
                                    cfga.id = ucfv.custom_field_group_assignment_id
                                left join flow.custom_field cf on
                                    cf.id = cfga.custom_field_id
                                left join flow.user u on u.id = ucfv.int_value
                                where 
                                    cfga.id = 330
                                    and 
                                    cf.id = 455
                            )
                            select 
                                DISTINCT u.id,
                                u.first_name,
                                u.last_name,
                                ust.user_status_type,
                                pos.position,
                                up.start_date,
                                up.end_date,
                                (uphv.hierarchy::json -> 4)::json ->> 'orgName' "area",
                                (uphv.hierarchy::json -> 4)::json ->> 'orgId' "area_id",
                                (uphv.hierarchy::json -> 4)::json ->> 'level' "area_level",
                                (uphv.hierarchy::json -> 5)::json ->> 'orgName' "region", -- THIS is the manager
                                (uphv.hierarchy::json -> 5)::json ->> 'orgId' "region_id",
                                (uphv.hierarchy::json -> 5)::json ->> 'level' "region_level",
                                (uphv.hierarchy::json -> 6)::json ->> 'orgName' "district",
                                (uphv.hierarchy::json -> 6)::json ->> 'orgId' "district_id",
                                (uphv.hierarchy::json -> 6)::json ->> 'level' "district_level",
                                (uphv.hierarchy::json -> 7)::json ->> 'orgName' "office",
                                (uphv.hierarchy::json -> 7)::json ->> 'orgId' "office_id",
                                (uphv.hierarchy::json -> 7)::json ->> 'level' "office_level",
                                rb.recruiter_user_id "recruiter_user_id",
                                rb.recruiter_name "recruiter_name",
                                u.email "user_email",
                                u_rec.email "recruiter_email"

                            from flow.user u

                            inner join flow.user_position up on 
                                (up.user_id = u.id
                                and
                                up.primary_flag is True)

                            inner join flow.position pos on
                                pos.id = up.position_id

                            inner join flow.company com on
                                com.id = pos.company_id

                            inner join flow.org on
                                org.id = up.org_id

                            inner join flow.org_type ot on
                                ot.id = org.org_type_id

                            inner join flow.user_positions_vw upv
                                on upv.user_id = u.id

                            inner join flow.user_status_type ust
                                on ust.id = upv.user_status_type_id

                            inner join flow.user_position_hierarchy_vw uphv on
                                uphv.user_id = u.id

                            left join recruited_by rb on
                                rb.user_id = u.id

                            left join flow.user u_rec on
                                u_rec.id = rb.recruiter_user_id

                            where                                 
                                (ust.user_status_type = 'New Request'
                                and
                                (
                                    pos.position = 'Closer' 
                                    or 
                                    pos.position = 'Setter'
                                )
                                and
                                up.start_date is not null
                                and
                                up.end_date is null
                                and
                                up.archived is False
                                and
                                up.primary_flag is True)
                    )
                    Select 
                        row_to_json(NEW_SALES_USERS.*) "json_info_new_sales_user"

                    from 
                        NEW_SALES_USERS
                """
    
        return all_users_info_query_string

class TaxDocConfigs:
    web_driver_hidden = True
