import os

try:
    import base64
except:
    os.system('pip install base64')
    import base64

try:
    import mimetypes
except:
    os.system('pip install mimetypes')
    import mimetypes

try:
    import regex as re
except:
    os.system('pip install regex')
    import regex as re

try:
    from adal import AuthenticationContext
except:
    os.system('pip install adal')
    from adal import AuthenticationContext

try:
    import pyperclip
except:
    os.system('pip install pyperclip')
    import pyperclip

import urllib
import requests
import json
import webbrowser

from . import Auth


def support_shared_inbox_device_flow_session(client_id, auto=False):
    """Obtain an access token from Azure AD (via device flow) and create
    a Requests session instance ready to make authenticated calls to
    Microsoft Graph.

    client_id = Application ID for registered "Azure AD only" V1-endpoint app
    auto      = whether to copy device code to clipboard and auto-launch browser

    Returns Requests session object if user signed in successfully. The session
    includes the access token in an Authorization header.

    User identity must be an organizational account (ADAL does not support MSAs).
    """

    ctx = AuthenticationContext(Auth.AUTHORITY_URL, api_version=None)
    # token_response = ctx.acquire_token_with_username_password(mail_config.RESOURCE,
    #                                                           'support@blueravensolar.com',
    #                                                           'TheRaven2020', client_id
    #                                                           )

    token_response = ctx.acquire_token_with_username_password(Auth.RESOURCE,
                                                              'support@blueravensolar.com',
                                                              'BRAdmin321', client_id
                                                              )

    if not token_response.get('accessToken', None):
        return None

    session = requests.Session()
    session.headers.update({'Authorization': f'Bearer {token_response["accessToken"]}',
                            'SdkVersion': 'sample-python-adal',
                            'x-client-SKU': 'sample-python-adal'})
    return session

def send_mail_from_support_shared_inbox(session, *, subject, recipients, body='', content_type='HTML',
              attachments=None):
    """Send email from support inbox

    session      = requests.Session() instance with Graph access token
    subject      = email subject (required)
    recipients   = list of recipient email addresses (required)
    body         = body of the message
    content_type = content type (default is 'HTML')
    attachments  = list of file attachments (local filenames)

    Returns the response from the POST to the sendmail API.
    """

    # Create recipient list in required format.

    to_recipient_list = recipients['to_recipients']
    #print(to_recipient_list)
    cc_recipient_list = recipients['cc_recipients']
    #print(cc_recipient_list)
    bcc_recipient_list = recipients['bcc_recipients']
    #print(bcc_recipient_list)

    # Create 'Sent From' address in required format.
    sent_from = {
        "emailAddress": {
            "address": "support@blueravensolar.com",
            "name": "Blue Raven Solar Customer Support"
        }
    }

    # Create list of attachments in required format.
    attached_files = []
    if attachments:
        for filename in attachments:
            b64_content = base64.b64encode(open(filename, 'rb').read())
            mime_type = mimetypes.guess_type(filename)[0]
            mime_type = mime_type if mime_type else ''
            attached_files.append(
                {'@odata.type': '#microsoft.graph.fileAttachment',
                 'ContentBytes': b64_content.decode('utf-8'),
                 'ContentType': mime_type,
                 'Name': filename})

    # Create email message in required format.
    email_msg = {'Message': {'Subject': subject,
                             'Body': {'ContentType': content_type, 'Content': body},
                             'toRecipients': to_recipient_list,
                             'ccRecipients': cc_recipient_list,
                             'bccRecipients': bcc_recipient_list,
                             'From': sent_from,
                             'Attachments': attached_files},

                 'SaveToSentItems': 'true',
                 }

    # Do a POST to Graph's sendMail API and return the response.
    return session.post(api_endpoint('me/microsoft.graph.sendMail'),
                        headers={'Content-Type': 'application/json'},
                        json=email_msg)

def saleshr_device_flow_session(client_id, auto=False):
    """Obtain an access token from Azure AD (via device flow) and create
    a Requests session instance ready to make authenticated calls to
    Microsoft Graph.

    client_id = Application ID for registered "Azure AD only" V1-endpoint app
    auto      = whether to copy device code to clipboard and auto-launch browser

    Returns Requests session object if user signed in successfully. The session
    includes the access token in an Authorization header.

    User identity must be an organizational account (ADAL does not support MSAs).
    """
    ctx = AuthenticationContext(Auth.AUTHORITY_URL_alt, api_version=None)
    token_response = ctx.acquire_token_with_username_password(Auth.RESOURCE_alt, 'ariel.randall@blueravensolar.com',
                                                              'Solar101!', client_id)
    # token_response = ctx.acquire_token_with_username_password(taxdocsconfig.RESOURCE, 'jeffrey.allan@blueravensolar.com','BlueRaven101',client_id)
    # print(token_response)
    '''
    device_code = ctx.acquire_user_code(taxdocsconfig.RESOURCE,
                                        client_id)

    # display user instructions
    if auto:
        pyperclip.copy(device_code['user_code']) # copy user code to clipboard
        webbrowser.open(device_code['verification_url']) # open browser
        print(f'The code {device_code["user_code"]} has been copied to your clipboard, '
              f'and your web browser is opening {device_code["verification_url"]}. '
              'Paste the code to sign in.')
    else:
        print(device_code['message'])


    token_response = ctx.acquire_token_with_device_code(taxdocsconfig.RESOURCE,
                                                        device_code,
                                                       client_id)
    print(token_response)

    '''
    if not token_response.get('accessToken', None):
        return None

    session = requests.Session()
    session.headers.update({'Authorization': f'Bearer {token_response["accessToken"]}',
                            'SdkVersion': 'sample-python-adal',
                            'x-client-SKU': 'sample-python-adal'})
    return session

def send_sales_mail(session, *, subject, recipients, body='', content_type='HTML',
              attachments=None):
    """Send email from current user.

    session      = requests.Session() instance with Graph access token
    subject      = email subject (required)
    recipients   = list of recipient email addresses (required)
    body         = body of the message
    content_type = content type (default is 'HTML')
    attachments  = list of file attachments (local filenames)

    Returns the response from the POST to the sendmail API.
    """

    # Create recipient list in required format.
    recipient_list = [{'EmailAddress': {'Address': address}}
                      for address in recipients]

    # Create list of attachments in required format.
    attached_files = []
    if attachments:
        for filename in attachments:
            b64_content = base64.b64encode(open(filename, 'rb').read())
            mime_type = mimetypes.guess_type(filename)[0]
            mime_type = mime_type if mime_type else ''
            attached_files.append(
                {'@odata.type': '#microsoft.graph.fileAttachment',
                 'ContentBytes': b64_content.decode('utf-8'),
                 'ContentType': mime_type,
                 'Name': filename})

    # Create email message in required format.
    email_msg = {'Message': {'Subject': subject,
                             'Body': {'ContentType': content_type, 'Content': body},
                             'ToRecipients': recipient_list,
                             "from": {
                                 "emailAddress": {
                                     "address": "saleshr@blueravensolar.com"
                                 }
                             },
                             'Attachments': attached_files},
                 'SaveToSentItems': 'true',
                 }



    # Do a POST to Graph's sendMail API and return the response.
    return session.post(api_endpoint('me/microsoft.graph.sendMail'),
                        headers={'Content-Type': 'application/json'},
                        json=email_msg)

def send_mail(session, *, subject, recipients, body='', content_type='HTML',
              attachments=None):
    """Send email from current user.

    session      = requests.Session() instance with Graph access token
    subject      = email subject (required)
    recipients   = list of recipient email addresses (required)
    body         = body of the message
    content_type = content type (default is 'HTML')
    attachments  = list of file attachments (local filenames)

    Returns the response from the POST to the sendmail API.
    """

    to_recipient_list = [
        {
            "emailAddress": {
                "address": "" + to_recipient["emailAddress"] + "",
                "Name": "" + to_recipient["name"] + ""
            }
        }
        for to_recipient in recipients["to_recipients"]
    ]

    cc_recipient_list = [
        {
            "emailAddress": {
                "address": "" + cc_recipient["emailAddress"] + "",
                "Name": "" + cc_recipient["name"] + ""
            }
        }
        for cc_recipient in recipients["cc_recipients"]
    ]

    bcc_recipient_list = [
        {
            "emailAddress": {
                "address": "" + bcc_recipient["emailAddress"] + "",
                "Name": "" + bcc_recipient["name"] + ""
            }
        }
        for bcc_recipient in recipients["bcc_recipients"]
    ]

    # Create 'Sent From' address in required format.
    sent_from = {
        "emailAddress": {
            "address": "support@blueravensolar.com",
            "name": "Blue Raven Solar Customer Support"
        }
    }

    # Create list of attachments in required format.
    attached_files = []
    if attachments:
        for filename in attachments:
            b64_content = base64.b64encode(open(filename, 'rb').read())
            mime_type = mimetypes.guess_type(filename)[0]
            mime_type = mime_type if mime_type else ''
            attached_files.append(
                {'@odata.type': '#microsoft.graph.fileAttachment',
                 'ContentBytes': b64_content.decode('utf-8'),
                 'ContentType': mime_type,
                 'Name': filename})
    # print()
    # Create email message in required format.
    email_msg = {'Message': {'Subject': subject,
                             'Body': {'ContentType': content_type, 'Content': body},
                             'ToRecipients': to_recipient_list,
                             'ccRecipients': cc_recipient_list,
                             'bccRecipients': bcc_recipient_list,
                             # 'From': sent_from,
                             'Attachments': attached_files},

                 'SaveToSentItems': 'true',
                 }

    # print(email_msg)

    # Do a POST to Graph's sendMail API and return the response.
    return session.post(api_endpoint('me/microsoft.graph.sendMail'),
                        headers={'Content-Type': 'application/json'},
                        json=email_msg)

def api_endpoint(url):
    """Convert a relative path such as /me/photo/$value to a full URI based
    on the current RESOURCE and API_VERSION settings in taxdocsconfig.py.
    """
    if urllib.parse.urlparse(url).scheme in ['http', 'https']:
        return url # url is already complete
    return urllib.parse.urljoin(f'{Auth.RESOURCE}/{Auth.API_VERSION}/',
                                url.lstrip('/'))

def get_message(session):
    endpoint = "me/messages/Inbox"
    messagesresponse = session.get(api_endpoint(endpoint))
    messagesjson = messagesresponse.json()
    #print(type(messagesjson.get('value')))
    print(messagesjson)

    '''    while True:
        nextLink = messagesjson.get('@odata.nextLink')
        try:
            messagesresponse = session.get(api_endpoint(nextLink))
            messagesjson = messagesresponse.json()
            print(messagesjson)
        except:
            break'''


    for item in messagesjson.get('value'):
        #print(type(item))
        #print(item.get('id'))
        messageid = item.get('id')
        messageendpoint = f"/me/mailFolders('AQMkADk1MDFkNzVlLTA5ZTQtNDAzYi1iNDY4LWIyOWE3MmM3ZjA4ZQAuAAADnvidjaBuZUq-0MRWkjkrOwEAN95_px_XL0WFw-VdwEZDBQACrPeESwAAAA==')/messages('{messageid}')?$select=body,subject"
        messageresponse = session.get(api_endpoint(messageendpoint),headers={'Perfer' : 'outlook.body-content-type = "text"'})
        messagejson = messageresponse.json()
        #print(type(messagejson))
        bodydict = messagejson.get('body')
        body = bodydict.get('content')
        #print(type(body))
        #print(body)
        subject = (messagejson.get('subject'))
        print(subject)
        r = re.compile("(\d{3}[-\.\s]??\d{3}[-\.\s]??\d{4}|\(\d{3}\)\s*\d{3}[-\.\s]??\d{4}|\d{3}[-\.\s]??\d{4})")
        phonenumber = r.findall(body)
        print(phonenumber)
        stringlocation = body.find("Jeff")
        print(f"Jeff location: {stringlocation}")

        if subject == "Bot Idea":
            destination = {'destinationID': 'AAMkADk1MDFkNzVlLTA5ZTQtNDAzYi1iNDY4LWIyOWE3MmM3ZjA4ZQAuAAAAAACe_J2NoG5lSr-QxFaSOSs7AQA33n6nH5cvRYXD9V3ARkMFAAKzRkbMAAA='}
            session.post(api_endpoint(f'me/messages/{messageid}/move'),
                                headers={'Content-Type': 'application/json'},
                                json=destination)

        #messageid = message.get('id')
        #print(messageid)
        #messagesubject = message.get('subject')
        #print(messagesubject)
    '''
    response = session.get(api_endpoint(endpoint))
    endpoint = "/me/mailFolders/'AQMkADk1MDFkNzVlLTA5ZTQtNDAzYi1iNDY4LWIyOWE3MmM3ZjA4ZQAuAAADnvidjaBuZUq-0MRWkjkrOwEAN95_px_XL0WFw-VdwEZDBQACrPeEPgAAAA=='/childFolders"
    messagesresponse = session.get(api_endpoint(endpoint))
    messagesjson = messagesresponse.json()
    print(messagesjson)
    '''
    #print(response)
    return

def get_code(session):
    endpoint = "me/mailFolders/Inbox/messages"
    messagesresponse = session.get(api_endpoint(endpoint))
    messagesjson = messagesresponse.json()
    #print(type(messagesjson.get('value')))
    print(messagesjson)
    code = "0"
    for item in messagesjson.get('value'):
        messageid = item.get('id')
        messageendpoint = f"/me/mailFolders/Inbox/messages('{messageid}')?$select=body,subject"
        messageresponse = session.get(api_endpoint(messageendpoint),headers={'Perfer' : 'outlook.body-content-type = "text"'})
        messagejson = messageresponse.json()
        if messagejson.get('subject') == "ACTION REQUIRED By User":
            bodydict = messagejson.get('body')
            body = bodydict.get('content')
            r = re.compile(r"(\d{6})")
            code = r.findall(body)
            print(code)
            destination = {
                'destinationID': 'AAMkADk1MDFkNzVlLTA5ZTQtNDAzYi1iNDY4LWIyOWE3MmM3ZjA4ZQAuAAAAAACe_J2NoG5lSr-QxFaSOSs7AQA33n6nH5cvRYXD9V3ARkMFAAKzRkbMAAA='}
            session.post(api_endpoint(f'me/messages/{messageid}/move'),
                         headers={'Content-Type': 'application/json'},
                         json=destination)

    return code

def sharing_link(session, *, item_id, link_type='view'):
    """Get a sharing link for an item in OneDrive.

    session   = requests.Session() instance with Graph access token
    item_id   = the id of the DriveItem (the target of the link)
    link_type = 'view' (default), 'edit', or 'embed' (OneDrive Personal only)

    Returns a tuple of the response object and the sharing link.
    """
    endpoint = f'me/drive/items/{item_id}/createLink'
    response = session.post(api_endpoint(endpoint),
                            headers={'Content-Type': 'application/json'},
                            json={'type': link_type})

    if response.ok:
        # status 201 = link created, status 200 = existing link returned
        return (response, response.json()['link']['webUrl'])
    return (response, '')

def upload_file(session, *, filename, folder=None):
    """Upload a file to OneDrive for Business.

    session  = requests.Session() instance with Graph access token
    filename = local filename; may include a path
    folder   = destination subfolder/path in OneDrive for Business
               None (default) = root folder

    File is uploaded and the response object is returned.
    If file already exists, it is overwritten.
    If folder does not exist, it is created.

    API documentation:
    https://developer.microsoft.com/en-us/graph/docs/api-reference/v1.0/api/driveitem_put_content
    """
    fname_only = os.path.basename(filename)

    # create the Graph endpoint to be used
    if folder:
        # create endpoint for upload to a subfolder
        endpoint = f'me/drive/root:/{folder}/{fname_only}:/content'
    else:
        # create endpoint for upload to drive root folder
        endpoint = f'me/drive/root/children/{fname_only}/content'

    content_type, _ = mimetypes.guess_type(fname_only)
    with open(filename, 'rb') as fhandle:
        file_content = fhandle.read()

    return session.put(api_endpoint(endpoint),
                       headers={'content-type': content_type},
                       data=file_content)